/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
** Copyright (C), 2003-2006, Victorian Partnership for Advanced Computing (VPAC) Ltd, 110 Victoria Street,
**	Melbourne, 3053, Australia.
** Copyright (c) 2005-2010, Monash University, Building 28, Monash University Clayton Campus,
**	Victoria, 3800, Australia
**
** Primary Contributing Organisations:
**	Victorian Partnership for Advanced Computing Ltd, Computational Software Development - http://csd.vpac.org
**	AuScope - http://www.auscope.org
**	Monash University, AuScope SAM VIC - http://www.auscope.monash.edu.au
**
** Contributors:
**	Robert Turnbull, Research Assistant, Monash University. (robert.turnbull@sci.monash.edu.au)
**	Patrick D. Sunter, Software Engineer, VPAC. (patrick@vpac.org)
**	Alan H. Lo, Computational Engineer, VPAC. (alan@vpac.org)
**	Stevan M. Quenette, Senior Software Engineer, VPAC. (steve@vpac.org)
**	David May, PhD Student, Monash University (david.may@sci.monash.edu.au)
**	Vincent Lemiale, Postdoctoral Fellow, Monash University. (vincent.lemiale@sci.monash.edu.au)
**	Julian Giordani, Research Assistant, Monash University. (julian.giordani@sci.monash.edu.au)
**	Louis Moresi, Associate Professor, Monash University. (louis.moresi@sci.monash.edu.au)
**	Luke J. Hodkinson, Computational Engineer, VPAC. (lhodkins@vpac.org)
**	Raquibul Hassan, Computational Engineer, VPAC. (raq@vpac.org)
**	David Stegman, Postdoctoral Fellow, Monash University. (david.stegman@sci.monash.edu.au)
**	Wendy Sharples, PhD Student, Monash University (wendy.sharples@sci.monash.edu.au)
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
**
**
**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

#define MAX_FOURIER_TERMS 45

#include <mpi.h>
#include <StGermain/StGermain.h>
#include <StgDomain/StgDomain.h>
#include <StgFEM/StgFEM.h>
#include <PICellerator/PICellerator.h>

#include <assert.h>

const Type AnalyticBlock_Type = "AnalyticBlock";

typedef struct {
	__AnalyticSolution
	FeVariable* velocityField;
	FeVariable* pressureField;
	double startBlockX;
	double endBlockX;
	double startBlockY;
} AnalyticBlock;

void _AnalyticBlock_VelocityFunction( void* analyticSolution, FeVariable* analyticFeVariable, double* coord, double* velocity ) {
	AnalyticBlock*          self           = (AnalyticBlock*)analyticSolution;
	double                  n;
	double                  x, y;
	double                  viscosity      = 1.0;
	double                  deltaRho       = 1.0;
	double                  startBlockX    = self->startBlockX;
	double                  endBlockX      = self->endBlockX;
	double                  startBlockY    = self->startBlockY;
	double                  x0;
	double                  dx;
	double                  u1, u2;
	double                  u1a, u1b;
	double                  u2a, u2b;
	double t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13,t14,t15,t16,t17,t18,t19,t20,t21,t22,t23,t24,t25,t26,t27,t28,t29,t30,t31,t32,t33,t34,t35,t36,t37,t38,t39,t40;
	double t41,t42,t44,t45,t46,t47,t48,t49,t50,t52,t53,t54,t55,t56,t57,t58,t59,t60,t61,t62,t63,t64,t65,t66,t67,t68,t69,t70,t71,t72,t73,t74,t75,t76,t77,t78,t79,t80;
	double t81,t82,t83,t84,t85,t86,t87,t88,t90,t91,t92,t93,t95,t96,t97,t98,t99,t100,t101,t104,t105,t106,t107,t108,t109,t110,t111,t112,t113,t115,t116,t117,t118,t119,t120;
	double t121,t122,t123,t124,t125,t126,t127,t128,t129,t130,t131,t132,t133,t135,t136,t137,t138,t140,t141,t142,t143,t144,t145,t146,t149,t150,t151,t153,t154,t155,t156,t157,t158,t159,t160;
	double t161,t162,t163,t165,t166,t167,t168,t169,t170,t171,t172,t173,t175,t176,t177,t178,t179,t180,t181,t182,t183,t185,t186,t187,t188,t189,t190,t191,t192,t193,t194,t195,t197,t198,t199;
	double t202,t203,t204,t206,t207,t208,t209,t210,t211,t212,t213,t214,t215,t216,t217,t218,t220,t221,t224,t225,t226,t227,t229,t230,t232,t233,t235,t236,t238,t239,t240;
	double t241,t243,t244,t247,t248,t249,t250,t251,t252,t253,t254,t255,t256,t258,t259,t260,t262,t263,t264,t265,t266,t268,t270,t271,t272,t273,t275,t276,t279,t280;
	double t281,t282,t283,t284,t285,t286,t287,t288,t290,t291,t293,t294,t295,t296,t297,t298,t299,t300,t301,t303,t304,t305,t307,t308,t309,t310,t313,t314,t316,t317,t319;
	double t321,t322,t323,t326,t327,t328,t330,t333,t334,t335,t337,t338,t339,t341,t342,t343,t344,t346,t347,t348,t349,t351,t353,t355,t357,t360;
	double t364,t365,t370,t371,t372,t373,t374,t379,t383,t384,t387,t388,t390,t394,t395,t396;
	double t401,t402,t403,t404,t406,t408,t409,t410,t411,t413,t414,t416,t421,t424,t425,t429,t430,t431,t435;
	double t441,t444,t446,t447,t449,t450,t453,t455,t459,t460,t462,t463,t464,t465,t467,t469,t471,t474,t479;
	double t481,t483,t484,t485,t490,t491,t492,t494,t496,t501,t507,t512,t518,t519;
	double t521,t522,t526,t535,t540,t542,t550,t557,t559;
	double t567,t569,t572,t577,t582,t587,t594,t598,t600;
	double t609,t610,t611,t613,t614,t615,t616,t618,t623,t628,t631,t634,t636,t637,t639,t640;
	double t641,t642,t645,t650,t654,t658,t659,t661,t670,t673;
	double t682,t685,t688,t701,t702,t705,t713,t716,t717,t719;
	double t721,t726,t728,t731,t734,t735,t740,t742,t744,t745,t747,t749,t753,t757;
	double t761,t771,t773,t779,t786;
	double t801,t803,t812,t813,t827,t830;
	double t853,t854,t863;
	double t883,t886,t892,t899,t907,t910,t917,t920;
	double t936,t947,t957;
	double t969,t973,t974,t978,t982,t997;
	double t1016;
	double t1041,t1051,t1059,t1072;
	double t1084,t1089,t1093,t1097;
	
	/* Find coordinate of node */
	x = coord[ I_AXIS ];
	y = coord[ J_AXIS ];

	x0 = 0.0;
	dx = endBlockX - startBlockX;

	/* Initialise */
	velocity[ I_AXIS ] = 0.0;
	velocity[ J_AXIS ] = 0.0;

	for( n = 1.0 ; n < MAX_FOURIER_TERMS ; n++ ) {
		t1 = n * M_PI;
		t2 = 0.1e1 / cosh(t1);
		t3 = startBlockY * n;
		t4 = t3 * M_PI;
		t5 = tanh(t4);
		t6 = t2 * t5;
		t7 = 0.1e1 / cosh(t4);
		t8 = tanh(t1);
		t9 = t8 * t7;
		t13 = sin(t1 * dx);
		t15 = y * n;
		t16 = t15 * M_PI;
		t17 = tanh(t16);
		t18 = t17 * t17;
		t19 = t18 * n;
		t20 = M_PI * t2;
		t21 = t20 * t8;
		t24 = t18 * t2;
		t25 = t8 * t5;
		t26 = t24 * t25;
		t29 = t2 * t2;
		t30 = t29 * t2;
		t32 = 0.1e1 / cosh(t16);
		t33 = t32 * t32;
		t35 = t33 * t8 * t7;
		t38 = t33 * t2;
		t39 = t38 * t5;
		t41 = t18 * t30;
		t42 = t41 * t5;
		t45 = t17 * t8 * t7;
		t47 = t38 * t8;
		t52 = t24 * startBlockY;
		t53 = t8 * t8;
		t54 = t53 * t5;
		t55 = t1 * t54;
		t57 = M_PI * t33;
		t60 = t53 * t8;
		t62 = t18 * t60 * t7;
		t64 = -0.4e1 * t19 * t21 + 0.2e1 * t16 * t26 + 0.2e1 * t30 - 0.3e1 * t16 * t35 + t16 * t39 - t16 * t42 + 0.12e2 * t45 + 0.2e1 * t16 * t47 - 0.2e1 * t33 * t7 + t52 * t55 - t15 * t57 * t30 - t16 * t62;
		t65 = t38 * startBlockY;
		t66 = t1 * t8;
		t71 = t33 * t30;
		t73 = t19 * M_PI;
		t74 = startBlockY * t29;
		t75 = t74 * t7;
		t77 = t74 * t9;
		t80 = t1 * t5;
		t82 = t17 * t30;
		t84 = M_PI * t18;
		t90 = t33 * n;
		t91 = t90 * M_PI;
		t93 = t29 * t8 * t7;
		t96 = t17 * n;
		t98 = M_PI * t29 * t7;
		t101 = -0.2e1 * t65 * t66 + 0.4e1 * t17 * t7 - 0.2e1 * t71 - t73 * t75 - t73 * t77 - 0.2e1 * t24 + t52 * t80 + 0.4e1 * t82 - t15 * t84 * t7 - t41 * startBlockY * t80 - 0.2e1 * t2 - 0.2e1 * t91 * t93 + 0.4e1 * t96 * t98;
		t104 = t1 * t53;
		t106 = t2 * t53;
		t107 = t106 * t5;
		t112 = t2 * t8;
		t113 = t112 * t5;
		t116 = t17 * t2;
		t117 = t116 * t53;
		t121 = t116 * t5;
		t124 = t1 * t25;
		t129 = t18 * startBlockY;
		t130 = t1 * t7;
		t132 = -0.2e1 * t6 - t65 * t104 - 0.2e1 * t73 * t107 + 0.4e1 * t90 * t21 - 0.4e1 * t73 * t113 - 0.4e1 * t117 + 0.4e1 * t47 - 0.6e1 * t35 + 0.2e1 * t16 * t121 + 0.2e1 * t52 * t124 + 0.2e1 * t19 * t98 + t129 * t130;
		t133 = t24 * t5;
		t137 = t20 * t53;
		t140 = t17 * t29;
		t141 = t140 * t9;
		t143 = t38 * t53;
		t146 = t18 * t53 * t7;
		t150 = t20 * t5;
		t153 = t140 * t7;
		t158 = M_PI * t8;
		t159 = t158 * t7;
		t162 = t16 * t133 + 0.2e1 * t15 * t21 + 0.2e1 * t90 * t137 - 0.4e1 * t141 + t16 * t143 + 0.6e1 * t146 + 0.2e1 * t42 - 0.2e1 * t93 - 0.2e1 * t19 * t150 + 0.2e1 * t16 * t153 + 0.2e1 * t38 + t15 * t150 - 0.3e1 * t15 * t159;
		t165 = t129 * n;
		t167 = M_PI * t60 * t7;
		t173 = t38 * t54;
		t179 = M_PI * t53;
		t180 = t179 * t7;
		t183 = t24 * t8;
		t187 = t33 * t29;
		t188 = t187 * t9;
		t190 = t17 * startBlockY;
		t191 = t190 * n;
		t195 = t165 * t167 + 0.2e1 * t73 * t93 + 0.2e1 * t16 * t117 + 0.2e1 * t173 + 0.3e1 * t165 * t159 - t15 * t84 * t30 + 0.3e1 * t165 * t180 + 0.2e1 * t16 * t183 + 0.2e1 * t7 + 0.2e1 * t188 + 0.2e1 * t191 * t167 + 0.2e1 * t39;
		t197 = t33 * t60 * t7;
		t199 = t187 * t7;
		t204 = t33 * t53 * t7;
		t207 = t33 * startBlockY;
		t208 = t207 * n;
		t213 = t116 * t8;
		t216 = t24 * t53;
		t221 = t17 * t60 * t7;
		t224 = -0.2e1 * t197 + 0.2e1 * t199 - 0.2e1 * t19 * t137 - 0.3e1 * t16 * t204 - t208 * t167 - 0.3e1 * t208 * t159 + 0.6e1 * t9 + 0.4e1 * t16 * t213 + t16 * t216 - 0.4e1 * t121 - 0.4e1 * t153 + 0.4e1 * t221 + 0.2e1 * t41;
		t227 = t71 * t5;
		t238 = t17 * t53 * t7;
		t240 = t82 * t5;
		t247 = -0.8e1 * t213 - 0.2e1 * t227 - 0.6e1 * t204 + 0.6e1 * t191 * t159 + t16 * t188 - 0.6e1 * t16 * t45 + t15 * t84 * t2 + 0.12e2 * t238 + 0.4e1 * t240 - 0.2e1 * t65 * t124 - t65 * t55 - 0.3e1 * t208 * t180;
		t248 = t96 * M_PI;
		t254 = M_PI * t17;
		t262 = t116 * startBlockY;
		t266 = t38 * t25;
		t271 = t29 * t7;
		t275 = -0.4e1 * t248 * t107 - 0.2e1 * t106 + t91 * t75 - 0.4e1 * t112 + 0.2e1 * t15 * t254 * t2 - 0.8e1 * t248 * t113 + 0.6e1 * t191 * t180 + 0.2e1 * t262 * t55 + t16 * t173 + 0.2e1 * t16 * t266 + 0.2e1 * t90 * t20 - 0.2e1 * t271 + 0.2e1 * t262 * t80;
		t283 = t18 * t29;
		t284 = t283 * t9;
		t293 = t2 * startBlockY;
		t300 = -0.3e1 * t16 * t146 - 0.2e1 * t248 * t75 + t16 * t284 + 0.4e1 * t262 * t124 - t15 * t57 * t7 + t15 * t57 * t2 + t16 * t93 + 0.2e1 * t293 * t66 - t16 * t227 + t293 * t104 + t16 * t199 + t16 * t107;
		t305 = t24 * t54;
		t313 = t283 * t7;
		t322 = -0.2e1 * t248 * t77 + 0.2e1 * t60 * t7 + t16 * t305 - 0.2e1 * t1 * t2 + 0.2e1 * t30 * t5 + t24 * t4 - 0.4e1 * t113 + t16 * t313 + 0.6e1 * t53 * t7 + t91 * t77 - t1 * t75 + 0.2e1 * t16 * t113 - t4 * t93;
		t327 = M_PI * t30;
		t335 = t30 * startBlockY;
		t347 = 0.4e1 * t248 * t93 + t15 * t98 - t15 * t327 * t5 - 0.4e1 * t96 * t137 + 0.2e1 * t116 * t4 - t16 * t197 - t335 * t80 - 0.2e1 * t82 * startBlockY * t80 - 0.2e1 * t90 * t98 - t41 * t4 - 0.2e1 * t1 * t107 - 0.2e1 * t16 * t221;
		t360 = t116 * t25;
		t370 = 0.2e1 * t1 * t93 + t3 * t167 - 0.6e1 * t16 * t238 + 0.2e1 * t18 * t7 + t52 * t104 + 0.2e1 * t52 * t66 + 0.3e1 * t3 * t159 + 0.4e1 * t16 * t360 + 0.3e1 * t3 * t180 - t15 * t167 - 0.2e1 * t107 + 0.2e1 * t143 - 0.2e1 * t82 * t4;
		t373 = t293 * n;
		t384 = t18 * t8 * t7;
		t395 = t373 * t179 * t5 - 0.4e1 * t1 * t113 + 0.2e1 * t16 * t141 + t293 * t80 - 0.4e1 * t183 - 0.2e1 * t216 + 0.6e1 * t384 + 0.2e1 * t373 * t158 * t5 - 0.3e1 * t15 * t180 + 0.2e1 * t262 * t104 - t65 * t80 - 0.2e1 * t133;
		t408 = t116 * t54;
		t414 = M_PI * t7;
		t416 = -0.2e1 * t313 + 0.2e1 * t62 - t207 * t130 - 0.3e1 * t16 * t384 - 0.2e1 * t305 - 0.2e1 * t284 - 0.2e1 * t19 * t20 - 0.8e1 * t360 - 0.2e1 * t16 * t240 + 0.2e1 * t16 * t408 - 0.4e1 * t26 - 0.2e1 * t1 * t6 + t3 * t414;
		t435 = -t335 * t1 + t293 * t1 - 0.2e1 * t1 * t106 - 0.4e1 * t1 * t112 + 0.2e1 * t1 * t271 + 0.2e1 * t190 * t130 - t15 * t327 + 0.4e1 * t266 - t15 * t414 - t38 * t4 - 0.4e1 * t408 - 0.4e1 * t96 * t20;
		t460 = t15 * t20 - 0.2e1 * t15 * t254 * t30 + t71 * startBlockY * t80 + 0.2e1 * t90 * t150 - 0.2e1 * t15 * t254 * t7 - 0.8e1 * t96 * t21 + t15 * t137 - 0.4e1 * t96 * t150 + 0.2e1 * t91 * t107 + 0.4e1 * t262 * t66 + t71 * t4 + 0.4e1 * t91 * t113 - 0.4e1 * t116;
		t471 = n * n;
		t474 = M_PI * M_PI;
		t479 = pow(t2 + 0.1e1 + t8, 0.2e1);
		t483 = pow(t2 - 0.1e1 - t8, 0.2e1);

		u1a = (t2 + t6 - t7 - t9) * deltaRho * t13 * (t64 + t101 + t132 + t162 + t195 + t224 + t247 + t275 + t300 + t322 + t347 + t370 + t395 + t416 + t435 + t460) / t7 / (0.1e1 + t5) / t471 / n / t474 / M_PI / t479 / t483 / t32 / viscosity / (0.1e1 + t17) / 0.2e1;

		t1 = n * M_PI;
		t3 = sin(t1 * dx);
		t5 = y * n;
		t6 = t5 * M_PI;
		t7 = 0.1e1 / cosh(t6);
		t8 = startBlockY * n;
		t9 = t8 * M_PI;
		t10 = 0.1e1 / cosh(t9);
		t11 = t7 * t10;
		t12 = tanh(t1);
		t13 = t12 * t12;
		t14 = t13 * t13;
		t15 = tanh(t9);
		t19 = tanh(t6);
		t20 = t19 * t19;
		t21 = t20 * startBlockY;
		t22 = t21 * n;
		t23 = M_PI * t12;
		t24 = 0.1e1 / cosh(t1);
		t25 = t24 * t24;
		t26 = t15 * t25;
		t27 = t23 * t26;
		t30 = t20 * t24;
		t31 = t12 * t10;
		t32 = t31 * t15;
		t35 = t7 * t7;
		t36 = t15 * t15;
		t37 = t35 * t36;
		t38 = t25 * t13;
		t39 = t37 * t38;
		t41 = t25 * t12;
		t42 = t37 * t41;
		t44 = t25 * t24;
		t45 = t19 * t44;
		t46 = t45 * t31;
		t48 = t19 * t24;
		t49 = t10 * t15;
		t50 = t48 * t49;
		t52 = t35 * startBlockY;
		t55 = t25 * t25;
		t56 = t55 * t15;
		t59 = t19 * t15;
		t60 = t59 * t41;
		t62 = t48 * t10;
		t65 = t10 * t10;
		t66 = t35 * t65;
		t67 = t66 * t38;
		t69 = t19 * t36;
		t70 = t69 * t38;
		t73 = t66 * t41;
		t75 = -0.4e1 * t11 * t14 * t15 + 0.4e1 * t22 * t27 + 0.12e2 * t30 * t32 - 0.2e1 * t39 - 0.4e1 * t42 - 0.8e1 * t46 + 0.8e1 * t50 - t52 * t1 * t36 - 0.4e1 * t11 * t56 - 0.16e2 * t60 - 0.4e1 * t6 * t62 - 0.2e1 * t67 + 0.2e1 * t6 * t70 - 0.4e1 * t73;
		t79 = t35 * t24;
		t80 = t13 * t10;
		t81 = t79 * t80;
		t83 = t69 * t41;
		t85 = t19 * n;
		t86 = t85 * M_PI;
		t87 = t24 * t12;
		t88 = t87 * t49;
		t91 = t52 * n;
		t92 = M_PI * t65;
		t93 = t92 * t25;
		t95 = t20 * n;
		t97 = M_PI * t44 * t10;
		t100 = t35 * n;
		t101 = t100 * M_PI;
		t106 = t13 * t12;
		t107 = t24 * t106;
		t108 = t107 * t49;
		t111 = t20 * t44;
		t112 = t111 * t31;
		t115 = t7 * t19;
		t116 = t10 * t14;
		t119 = t87 * t10;
		t122 = t19 * t65;
		t123 = t122 * t41;
		t126 = 0.8e1 * t11 * t38 + 0.2e1 * t37 - 0.12e2 * t81 - 0.8e1 * t83 + 0.12e2 * t86 * t88 - t91 * t93 + 0.2e1 * t95 * t97 - 0.6e1 * t101 * t88 + 0.8e1 * t11 * t26 - 0.2e1 * t101 * t108 + 0.2e1 * t6 * t112 - 0.4e1 * t115 * t116 + 0.6e1 * t1 * t119 - 0.8e1 * t123 - 0.4e1 * t70;
		t128 = t95 * M_PI;
		t129 = t44 * t10;
		t130 = t129 * t15;
		t133 = t30 * t31;
		t138 = t10 * t106;
		t141 = M_PI * t35;
		t142 = t5 * t141;
		t143 = t24 * t13;
		t144 = t143 * t49;
		t150 = t35 * t15;
		t151 = t150 * t41;
		t154 = t69 * t25;
		t157 = M_PI * t19;
		t158 = t5 * t157;
		t168 = t20 * t55;
		t169 = t168 * t65;
		t171 = 0.2e1 * t128 * t130 + 0.12e2 * t133 - 0.4e1 * t91 * t23 * t36 - 0.16e2 * t115 * t138 - 0.6e1 * t142 * t144 - 0.4e1 * t41 + 0.4e1 * t91 * t27 - 0.4e1 * t6 * t151 + 0.2e1 * t6 * t154 - 0.12e2 * t158 * t144 - 0.24e2 * t11 * t13 * t15 - 0.2e1 * t101 * t130 - 0.6e1 * t6 * t81 - t6 * t169;
		t172 = t26 * t12;
		t175 = M_PI * t13;
		t176 = t175 * t25;
		t178 = M_PI * t55;
		t179 = t178 * t36;
		t181 = t178 * t15;
		t185 = t44 * t12;
		t186 = t185 * t10;
		t188 = t35 * t14;
		t189 = t188 * t36;
		t191 = t35 * t106;
		t192 = t191 * t36;
		t194 = t19 * t25;
		t195 = t194 * t12;
		t197 = t35 * t25;
		t198 = t197 * t12;
		t202 = M_PI * t36;
		t203 = t202 * t25;
		t211 = -0.16e2 * t86 * t172 + t91 * t176 - t8 * t179 - 0.2e1 * t8 * t181 - 0.4e1 * t130 - 0.4e1 * t186 + 0.2e1 * t189 + 0.8e1 * t192 - 0.8e1 * t195 - 0.4e1 * t198 - 0.2e1 * t101 * t186 + t91 * t203 + t5 * t141 * t36 - 0.6e1 * t6 * t119 + t5 * t141 * t65;
		t214 = t35 * t13;
		t215 = t214 * t65;
		t226 = t19 * startBlockY;
		t227 = t226 * n;
		t230 = t20 * t25;
		t232 = t20 * t36;
		t233 = t232 * t41;
		t235 = M_PI * t25;
		t238 = t20 * t15;
		t239 = t238 * t41;
		t243 = t20 * t65;
		t244 = t243 * t41;
		t247 = t243 * t38;
		t251 = t36 * t25;
		t252 = t251 * t13;
		t255 = 0.6e1 * t6 * t215 + 0.6e1 * t5 * t141 * t13 + 0.2e1 * t128 * t186 - 0.16e2 * t11 * t12 * t15 - 0.2e1 * t227 * t179 - 0.2e1 * t230 - 0.4e1 * t233 + 0.2e1 * t100 * t235 - 0.8e1 * t239 - 0.6e1 * t101 * t144 + 0.2e1 * t6 * t244 + t6 * t247 - 0.6e1 * t101 * t119 + 0.2e1 * t101 * t252;
		t259 = t235 * t15;
		t265 = t35 * t44;
		t266 = t265 * t31;
		t279 = t80 * t15;
		t282 = t115 * t10;
		t283 = t26 * t13;
		t286 = t191 * t15;
		t290 = -0.2e1 * t247 - 0.2e1 * t95 * t176 + 0.4e1 * t100 * t259 - 0.4e1 * t26 + 0.24e2 * t48 * t32 + 0.2e1 * t6 * t266 + 0.4e1 * t6 * t239 + 0.2e1 * t5 * t141 * t15 + 0.2e1 * t66 - 0.2e1 * t6 * t108 - 0.2e1 * t128 * t252 + 0.12e2 * t30 * t279 + 0.8e1 * t282 * t283 + 0.8e1 * t6 * t286 + t22 * t203;
		t295 = M_PI * t24 * t10;
		t299 = t168 * t36;
		t303 = t24 * t10;
		t308 = t185 * t49;
		t310 = t303 * t15;
		t313 = t48 * t138;
		t316 = t202 * t41;
		t319 = M_PI * t20;
		t322 = t65 * t25;
		t323 = t322 * t12;
		t326 = t188 * t65;
		t330 = 0.2e1 * t9 * t283 + 0.2e1 * t95 * t295 + 0.8e1 * t191 - t6 * t299 + 0.2e1 * t227 * t203 + 0.2e1 * t1 * t303 - 0.2e1 * t22 * t181 - 0.4e1 * t308 - 0.2e1 * t101 * t310 - 0.4e1 * t6 * t313 + 0.2e1 * t91 * t316 + t5 * t319 * t25 + 0.4e1 * t101 * t323 + t6 * t326 - 0.2e1 * t100 * t295;
		t337 = t178 * t65;
		t346 = t30 * t138;
		t349 = t66 * t25;
		t351 = t143 * t10;
		t353 = t107 * t10;
		t355 = t230 * t13;
		t357 = t30 * t10;
		t360 = -0.4e1 * t1 * t283 + 0.2e1 * t55 * t36 - 0.2e1 * t101 * t308 - t5 * t337 - 0.2e1 * t5 * t295 + 0.4e1 * t108 + 0.4e1 * t6 * t46 + 0.2e1 * t128 * t310 - 0.2e1 * t6 * t346 - t6 * t349 + 0.12e2 * t351 + 0.4e1 * t353 - 0.2e1 * t355 + 0.4e1 * t357 + 0.2e1 * t326;
		t364 = t214 * t36;
		t370 = t168 * t15;
		t383 = t232 * t38;
		t387 = 0.12e2 * t364 + 0.8e1 * t11 * t25 + 0.2e1 * t299 + 0.4e1 * t310 + 0.4e1 * t370 - 0.4e1 * t11 * t55 + 0.4e1 * t85 * t97 - 0.4e1 * t1 * t26 + 0.12e2 * t144 - 0.2e1 * t1 * t251 + 0.2e1 * t6 * t233 + t6 * t383 - t8 * t178 + 0.4e1 * t56;
		t388 = t150 * t38;
		t396 = t238 * t38;
		t401 = t175 * t26;
		t406 = t138 * t15;
		t409 = M_PI * t106;
		t413 = t122 * t25;
		t421 = t92 * t41;
		t424 = -0.2e1 * t6 * t388 - 0.4e1 * t11 + 0.8e1 * t227 * t27 + 0.2e1 * t1 * t129 + 0.2e1 * t6 * t396 - 0.4e1 * t128 * t283 + 0.2e1 * t22 * t401 + 0.2e1 * t100 * t176 + 0.4e1 * t30 * t406 + 0.4e1 * t91 * t409 * t65 + 0.2e1 * t6 * t413 - 0.2e1 * t25 + 0.2e1 * t6 * t323 - 0.4e1 * t85 * t93 - 0.2e1 * t22 * t421;
		t431 = t322 * t13;
		t435 = t10 * t25;
		t446 = t79 * t49;
		t459 = -0.12e2 * t91 * t175 * t15 + 0.2e1 * t6 * t130 + t6 * t431 - 0.24e2 * t115 * t80 + 0.16e2 * t115 * t435 * t12 - t5 * t319 * t55 - 0.4e1 * t115 * t116 * t15 - 0.16e2 * t115 * t406 - 0.2e1 * t6 * t446 - 0.4e1 * t91 * t409 * t36 - 0.2e1 * t6 * t357 - t8 * t93 - 0.2e1 * t142 * t108 + 0.4e1 * t85 * t295;
		t462 = t111 * t10;
		t465 = t232 * t25;
		t469 = t30 * t49;
		t474 = t265 * t10;
		t481 = t188 * t15;
		t484 = startBlockY * t25;
		t485 = t484 * t12;
		t490 = t92 * t38;
		t492 = t202 * t38;
		t494 = t243 * t25;
		t496 = -0.2e1 * t6 * t73 + 0.2e1 * t6 * t462 + t6 * t465 + 0.4e1 * t101 * t283 - 0.2e1 * t6 * t469 - 0.12e2 * t158 * t88 + 0.2e1 * t6 * t474 - 0.16e2 * t115 * t32 + 0.2e1 * t6 * t308 + 0.2e1 * t6 * t481 + 0.2e1 * t1 * t485 + 0.16e2 * t282 * t172 - t91 * t490 + t91 * t492 + t6 * t494;
		t507 = t79 * t31;
		t521 = t235 * t12;
		t526 = -0.2e1 * t101 * t353 - 0.8e1 * t85 * t259 - 0.2e1 * t5 * t181 - 0.2e1 * t1 * t322 - 0.6e1 * t6 * t507 + 0.12e2 * t88 - 0.4e1 * t227 * t421 + t5 * t235 + 0.16e2 * t11 * t41 - 0.24e2 * t115 * t279 - 0.6e1 * t6 * t133 + t8 * t203 + 0.4e1 * t100 * t521 - 0.4e1 * t79 * t406;
		t535 = M_PI * t14;
		t542 = t59 * t25;
		t559 = 0.24e2 * t48 * t279 + 0.6e1 * t91 * t175 * t65 + 0.4e1 * t5 * t141 * t106 - 0.2e1 * t91 * t535 * t15 + 0.4e1 * t227 * t316 + t6 * t355 + 0.4e1 * t6 * t542 - 0.4e1 * t52 * t1 * t106 + 0.12e2 * t214 - 0.4e1 * t115 * t49 - 0.16e2 * t115 * t31 - 0.4e1 * t128 * t323 - t6 * t67 - 0.4e1 * t244 + 0.2e1 * t101 * t431;
		t569 = t111 * t49;
		t572 = t10 * t55;
		t577 = t194 * t13;
		t587 = t48 * t80;
		t594 = -0.2e1 * t322 - 0.4e1 * t52 * t1 * t12 + 0.4e1 * t6 * t123 - 0.8e1 * t86 * t323 + 0.2e1 * t6 * t569 - 0.4e1 * t115 * t572 + 0.2e1 * t22 * t316 + 0.2e1 * t6 * t577 + 0.2e1 * t168 - 0.2e1 * t6 * t370 - 0.4e1 * t86 * t252 - 0.12e2 * t79 * t32 - 0.12e2 * t6 * t587 - 0.2e1 * t1 * t431 + 0.2e1 * t1 * t353;
		t600 = t35 * t12;
		t610 = t265 * t49;
		t613 = t230 * t12;
		t616 = t5 * t319;
		t623 = 0.2e1 * t91 * t401 + 0.2e1 * t5 * t157 * t25 + 0.8e1 * t600 - 0.2e1 * t251 + t142 - 0.12e2 * t79 * t279 - 0.8e1 * t86 * t283 + 0.4e1 * t150 + 0.8e1 * t48 * t406 + 0.2e1 * t6 * t610 + 0.2e1 * t6 * t613 - 0.2e1 * t616 * t108 + 0.4e1 * t469 + t8 * t337 + 0.8e1 * t11 * t283;
		t628 = t37 * t25;
		t631 = t79 * t10;
		t636 = t30 * t80;
		t641 = t251 * t12;
		t650 = t197 * t13;
		t658 = -t6 * t628 + 0.4e1 * t346 - 0.2e1 * t6 * t631 + 0.2e1 * t55 * t65 + 0.12e2 * t636 - 0.2e1 * t5 * t157 * t55 - 0.4e1 * t1 * t641 - 0.2e1 * t95 * t203 - 0.8e1 * t45 * t32 - 0.6e1 * t142 * t88 - t6 * t650 - 0.8e1 * t1 * t172 + 0.2e1 * t227 * t337 + 0.16e2 * t11 * t172;
		t659 = t122 * t38;
		t661 = t59 * t38;
		t670 = t238 * t25;
		t685 = -0.4e1 * t659 - 0.8e1 * t661 - 0.2e1 * t128 * t431 - t52 * t1 * t14 + t22 * t176 - 0.4e1 * t111 * t32 + 0.2e1 * t6 * t670 - t9 * t431 + 0.24e2 * t587 - 0.4e1 * t396 + 0.4e1 * t265 * t32 + 0.4e1 * t86 * t485 + 0.2e1 * t100 * t93 + 0.2e1 * t128 * t308 - t5 * t178;
		t688 = M_PI * startBlockY * t25;
		t701 = t45 * t49;
		t713 = t1 * t24;
		t716 = t95 * t688 - 0.8e1 * t91 * t23 * t15 + t5 * t141 * t14 - 0.2e1 * t383 - 0.4e1 * t6 * t50 + 0.4e1 * t266 - 0.8e1 * t85 * t521 - 0.8e1 * t701 + 0.2e1 * t128 * t353 - 0.8e1 * t86 * t641 + 0.4e1 * t86 * t108 - 0.2e1 * t9 * t323 + t52 * t1 * t65 + 0.2e1 * t713 * t406;
		t717 = t1 * t55;
		t721 = t150 * t25;
		t745 = -t21 * t717 + 0.8e1 * t6 * t60 - 0.2e1 * t6 * t721 + 0.4e1 * t101 * t641 - 0.6e1 * t91 * t175 * t36 + 0.2e1 * t1 * t130 + 0.2e1 * t9 * t641 - 0.8e1 * t151 + 0.4e1 * t610 + 0.4e1 * t6 * t192 + t8 * t176 + 0.8e1 * t115 * t435 + 0.2e1 * t227 * t176 + 0.2e1 * t616 * t308 + t5 * t203;
		t749 = t214 * t15;
		t757 = t45 * t10;
		t773 = -0.4e1 * t388 + 0.12e2 * t6 * t749 + 0.12e2 * t86 * t119 + 0.6e1 * t713 * t32 - 0.2e1 * t197 + 0.4e1 * t6 * t757 - 0.4e1 * t446 + 0.2e1 * t6 * t659 + t5 * t176 + 0.4e1 * t86 * t130 - 0.4e1 * t1 * t323 - 0.2e1 * t6 * t310 - 0.12e2 * t507 - 0.6e1 * t6 * t351;
		t786 = t79 * t138;
		t803 = 0.2e1 * t5 * t521 - 0.4e1 * t128 * t641 + 0.2e1 * t22 * t259 + 0.4e1 * t91 * t23 * t65 - 0.4e1 * t85 * t176 - t22 * t179 - 0.4e1 * t786 - 0.6e1 * t616 * t144 - t5 * t141 * t25 - 0.4e1 * t158 * t108 + 0.2e1 * t6 * t641 + t6 * t252 + 0.4e1 * t6 * t661 - 0.6e1 * t6 * t88 - 0.4e1 * t95 * t521;
		t812 = t19 * t55;
		t813 = t812 * t15;
		t827 = -0.4e1 * t323 - 0.2e1 * t431 - 0.2e1 * t465 - 0.2e1 * t494 - 0.4e1 * t670 - 0.4e1 * t154 - 0.4e1 * t413 + 0.8e1 * t813 - 0.4e1 * t11 * t14 - 0.16e2 * t11 * t106 - 0.24e2 * t11 * t13 - 0.4e1 * t11 * t15 - 0.16e2 * t11 * t12 - 0.8e1 * t542 + 0.12e2 * t119;
		t853 = -0.6e1 * t6 * t144 + t91 * t535 * t65 - 0.2e1 * t100 * t97 - 0.8e1 * t128 * t172 + 0.2e1 * t128 * t108 + 0.2e1 * t5 * t97 + 0.4e1 * t158 * t308 + t9 * t252 + 0.2e1 * t188 + t1 * t484 + 0.6e1 * t128 * t351 - 0.2e1 * t38 - 0.4e1 * t227 * t181 - 0.4e1 * t194 - 0.2e1 * t1 * t38;
		t863 = t600 * t36;
		t886 = -0.2e1 * t6 * t198 + 0.4e1 * t86 * t310 - 0.4e1 * t6 * t813 + 0.4e1 * t6 * t863 + 0.2e1 * t8 * t259 + 0.2e1 * t1 * t310 + 0.6e1 * t1 * t351 + 0.4e1 * t303 + 0.2e1 * t142 * t308 + 0.2e1 * t227 * t492 + 0.4e1 * t6 * t172 - 0.4e1 * t95 * t259 - 0.8e1 * t91 * t409 * t15 + 0.4e1 * t227 * t259;
		t892 = t191 * t65;
		t899 = t812 * t36;
		t907 = t812 * t65;
		t917 = t48 * t31;
		t920 = -0.16e2 * t11 * t106 * t15 - 0.2e1 * t6 * t353 + 0.4e1 * t6 * t892 + 0.12e2 * t86 * t351 + 0.6e1 * t128 * t88 - 0.2e1 * t6 * t899 + 0.8e1 * t115 * t435 * t13 + 0.4e1 * t6 * t83 - 0.2e1 * t6 * t907 + 0.2e1 * t35 - t6 * t39 + t5 * t93 + 0.2e1 * t128 * t485 + 0.12e2 * t86 * t144 - 0.12e2 * t6 * t917;
		t947 = 0.2e1 * t5 * t259 - 0.4e1 * t85 * t203 + 0.2e1 * t100 * t203 - 0.2e1 * t6 * t786 - 0.2e1 * t1 * t252 - 0.2e1 * t226 * t717 + 0.6e1 * t128 * t144 - 0.2e1 * t227 * t93 + 0.4e1 * t86 * t308 - 0.6e1 * t616 * t88 + t100 * t688 + 0.8e1 * t313 - 0.6e1 * t6 * t636 - 0.4e1 * t112;
		t973 = -0.4e1 * t85 * t235 + 0.24e2 * t917 + 0.2e1 * t91 * t259 + 0.4e1 * t86 * t186 - 0.4e1 * t569 - t22 * t93 + t6 * t189 - 0.6e1 * t52 * t1 * t13 + 0.4e1 * t9 * t172 + t22 * t337 - t22 * t490 - 0.2e1 * t52 * t1 * t15 + 0.2e1 * t6 * t186 - t52 * t1 + 0.2e1 * t6 * t283;
		t997 = 0.8e1 * t101 * t172 + t22 * t492 + 0.6e1 * t713 * t279 - 0.2e1 * t95 * t235 + 0.8e1 * t62 - 0.2e1 * t628 - 0.2e1 * t349 - 0.2e1 * t650 + 0.8e1 * t892 + 0.12e2 * t215 + 0.4e1 * t481 + 0.2e1 * t85 * t688 + 0.4e1 * t5 * t141 * t12 - 0.2e1 * t95 * t93;
		t1016 = -0.4e1 * t1 * t41 - t5 * t179 - 0.4e1 * t721 - 0.4e1 * t577 - 0.4e1 * t283 - 0.8e1 * t172 - 0.4e1 * t462 + 0.16e2 * t286 + 0.4e1 * t899 + 0.4e1 * t907 + 0.2e1 * t169 - 0.8e1 * t757 - 0.4e1 * t631 + 0.2e1 * t1 * t186 + 0.4e1 * t6 * t195;
		t1041 = t600 * t65;
		t1051 = -0.2e1 * t227 * t490 + 0.4e1 * t227 * t401 + 0.4e1 * t6 * t701 - 0.6e1 * t101 * t351 - t91 * t535 * t36 - 0.4e1 * t129 + 0.6e1 * t6 * t364 + 0.2e1 * t1 * t44 * t32 + 0.6e1 * t128 * t119 - 0.2e1 * t6 * t42 + 0.8e1 * t115 * t49 * t25 + 0.4e1 * t6 * t1041 - 0.2e1 * t91 * t421 - 0.4e1 * t115 * t572 * t15 + 0.4e1 * t86 * t353;
		t1059 = t600 * t15;
		t1072 = 0.2e1 * t55 + 0.2e1 * t101 * t485 + 0.4e1 * t812 - 0.4e1 * t86 * t431 - 0.4e1 * t613 + 0.16e2 * t1059 + 0.24e2 * t749 + 0.4e1 * t474 + 0.8e1 * t1041 + 0.8e1 * t863 - 0.2e1 * t1 * t25 - 0.4e1 * t282 - 0.2e1 * t252 - 0.4e1 * t641 + 0.8e1 * t6 * t1059;
		t1084 = pow(t24 + 0.1e1 + t12, 0.2e1);
		t1089 = pow(t24 - 0.1e1 - t12, 0.2e1);
		t1093 = n * n;
		t1097 = M_PI * M_PI;

		u1b = deltaRho * t3 * (t1051 + t459 + t387 + t330 + t658 + t496 + t947 + t126 + t594 + t886 + t803 + t920 + t75 + t973 + t997 + t171 + t211 + t716 + t360 + t424 + t1072 + t559 + t745 + t1016 + t623 + t853 + t773 + t255 + t526 + t827 + t290 + t685) / t10 / (0.1e1 + t15) / t1084 / t1089 / t7 / t1093 / n / t1097 / M_PI / viscosity / (0.1e1 + t19) / 0.2e1;
		
		t1 = n * M_PI;
		t2 = 0.1e1 / cosh(t1);
		t3 = startBlockY * n;
		t4 = t3 * M_PI;
		t5 = tanh(t4);
		t6 = t2 * t5;
		t7 = 0.1e1 / cosh(t4);
		t8 = tanh(t1);
		t9 = t8 * t7;
		t13 = sin(t1 * dx);
		t15 = y * n;
		t16 = t15 * M_PI;
		t17 = tanh(t16);
		t19 = t17 * t8 * t7;
		t22 = t17 * t2;
		t25 = t17 * n;
		t26 = M_PI * t2;
		t27 = t8 * t8;
		t28 = t26 * t27;
		t31 = t2 * startBlockY;
		t32 = t1 * t5;
		t34 = t17 * t17;
		t35 = t34 * t2;
		t37 = 0.1e1 / cosh(t16);
		t38 = t37 * t37;
		t39 = t38 * n;
		t40 = t39 * M_PI;
		t41 = t2 * t8;
		t42 = t41 * t5;
		t45 = t38 * t2;
		t46 = t27 * t5;
		t47 = t45 * t46;
		t48 = t2 * t2;
		t49 = t38 * t48;
		t50 = t49 * t9;
		t53 = t8 * t5;
		t54 = t22 * t53;
		t56 = t48 * t2;
		t57 = t17 * t56;
		t60 = t17 * t48;
		t61 = t60 * t9;
		t63 = 0.6e1 * t16 * t19 - 0.2e1 * t22 * t4 + 0.4e1 * t25 * t28 - t31 * t32 - t35 * t4 + 0.4e1 * t40 * t42 + t47 + t50 + 0.2e1 * t39 * t26 + 0.4e1 * t54 + 0.2e1 * t57 * t4 + 0.2e1 * t61;
		t64 = t34 * n;
		t67 = t34 * t56;
		t69 = t35 * t53;
		t71 = t35 * t46;
		t72 = t34 * t48;
		t73 = t72 * t9;
		t74 = t22 * startBlockY;
		t75 = t1 * t27;
		t78 = t48 * t7;
		t83 = M_PI * t7;
		t85 = t56 * startBlockY;
		t88 = t2 * t27;
		t93 = 0.2e1 * t64 * t26 + t67 * t4 + 0.2e1 * t69 + t71 + t73 - 0.2e1 * t74 * t75 - 0.2e1 * t1 * t78 + 0.2e1 * t1 * t6 - t3 * t83 + t85 * t1 - t31 * t1 + 0.2e1 * t1 * t88 + 0.4e1 * t1 * t41;
		t98 = t31 * n;
		t99 = t8 * M_PI;
		t104 = t17 * startBlockY;
		t105 = t1 * t7;
		t108 = M_PI * t56;
		t110 = t45 * t53;
		t113 = t22 * t46;
		t117 = t38 * t56;
		t118 = t117 * t5;
		t120 = t26 * t5;
		t123 = 0.4e1 * t25 * t26 - t15 * t26 - 0.2e1 * t98 * t99 * t5 + t15 * t83 - 0.2e1 * t104 * t105 + t15 * t108 + 0.2e1 * t110 + t85 * t32 + 0.2e1 * t113 + 0.2e1 * t16 * t110 - t16 * t118 + 0.4e1 * t25 * t120;
		t125 = t34 * t27 * t7;
		t128 = t48 * t8 * t7;
		t129 = t88 * t5;
		t130 = t45 * t27;
		t131 = t67 * t5;
		t133 = t49 * t7;
		t135 = M_PI * t38;
		t140 = t26 * t8;
		t143 = t104 * n;
		t144 = t27 * t8;
		t146 = M_PI * t144 * t7;
		t149 = t99 * t7;
		t153 = M_PI * t48 * t7;
		t156 = -0.3e1 * t125 + t128 + t129 + t130 - t131 - t15 * t28 + t16 * t133 - t15 * t135 * t7 + t15 * t135 * t2 - 0.2e1 * t15 * t140 - 0.2e1 * t143 * t146 - 0.6e1 * t143 * t149 - 0.2e1 * t39 * t153;
		t159 = t45 * startBlockY;
		t160 = t1 * t46;
		t162 = M_PI * t27;
		t163 = t162 * t7;
		t166 = t38 * startBlockY;
		t170 = M_PI * t25;
		t171 = startBlockY * t48;
		t172 = t171 * t7;
		t177 = t35 * t8;
		t182 = t171 * t9;
		t186 = t34 * t8 * t7;
		t191 = -t159 * t160 - 0.6e1 * t143 * t163 - t166 * t105 + 0.2e1 * t39 * t120 + 0.2e1 * t170 * t172 + 0.2e1 * t1 * t129 - 0.2e1 * t16 * t177 - 0.2e1 * t40 * t128 + 0.2e1 * t170 * t182 + 0.3e1 * t16 * t186 + t117 * t4 - t45 * t4;
		t192 = t35 * t5;
		t194 = t166 * n;
		t199 = t1 * t8;
		t206 = t57 * t5;
		t210 = t38 * t27 * t7;
		t217 = -t16 * t192 - t194 * t146 + 0.2e1 * t57 * startBlockY * t32 - 0.4e1 * t74 * t199 - t56 - 0.4e1 * t16 * t54 - 0.3e1 * t9 - 0.2e1 * t57 + 0.2e1 * t16 * t206 - 0.3e1 * t210 + 0.2e1 * t1 * t2 - 0.4e1 * t25 * t153 + t16 * t131;
		t225 = M_PI * t17;
		t236 = t38 * t8 * t7;
		t239 = t45 * t5;
		t241 = t45 * t8;
		t244 = -0.2e1 * t16 * t69 - 0.2e1 * t159 * t199 - 0.2e1 * t1 * t128 + 0.2e1 * t15 * t225 * t56 + t35 - 0.2e1 * t17 * t7 + 0.2e1 * t15 * t225 * t7 - t159 * t75 - 0.3e1 * t16 * t236 - t117 + t16 * t239 + 0.2e1 * t16 * t241;
		t249 = t64 * M_PI;
		t253 = t38 * t144 * t7;
		t258 = t22 * t5;
		t264 = -t38 * t7 + t6 - 0.3e1 * t194 * t163 + t40 * t172 + t249 * t172 - t3 * t146 - t16 * t253 - t15 * t135 * t56 - t159 * t32 + t2 - 0.2e1 * t16 * t258 + t249 * t182 + t67 * startBlockY * t32;
		t270 = t60 * t7;
		t275 = t17 * t144 * t7;
		t279 = t72 * t7;
		t282 = t34 * t144 * t7;
		t284 = t35 * t27;
		t286 = 0.4e1 * t64 * t140 - 0.2e1 * t16 * t270 + t16 * t130 + 0.2e1 * t16 * t275 + t40 * t182 - t16 * t279 + t16 * t282 + t284 - 0.3e1 * t186 + t192 + t279 - t282;
		t287 = t35 * startBlockY;
		t295 = t22 * t27;
		t299 = t22 * t8;
		t304 = t17 * t27 * t7;
		t307 = -0.2e1 * t287 * t199 + 0.2e1 * t249 * t129 - t34 * t7 + 0.4e1 * t249 * t42 + 0.2e1 * t295 + 0.2e1 * t241 - 0.3e1 * t236 + t239 - t253 + 0.4e1 * t299 - t118 - 0.2e1 * t64 * t153 + 0.6e1 * t16 * t304;
		t309 = t34 * startBlockY;
		t317 = M_PI * t34;
		t328 = t309 * n;
		t330 = -t309 * t105 + t117 * startBlockY * t32 - t7 + 0.4e1 * t39 * t140 + 0.2e1 * t64 * t120 + t45 + t15 * t317 * t7 - 0.2e1 * t249 * t128 - 0.3e1 * t3 * t149 + 0.2e1 * t39 * t28 - 0.3e1 * t194 * t149 - t328 * t146;
		t342 = t1 * t53;
		t349 = -0.3e1 * t328 * t149 + t15 * t317 * t56 - t67 - t16 * t284 - 0.3e1 * t328 * t163 + 0.2e1 * t64 * t28 - t15 * t317 * t2 - 0.2e1 * t159 * t342 - t287 * t160 + t133 + 0.2e1 * t258 + 0.2e1 * t270 - 0.2e1 * t275;
		t371 = -0.6e1 * t304 - t287 * t75 + 0.2e1 * t177 + 0.4e1 * t170 * t129 - 0.2e1 * t15 * t225 * t2 - t287 * t32 - 0.3e1 * t3 * t163 - 0.2e1 * t287 * t342 - 0.2e1 * t16 * t295 + 0.8e1 * t170 * t42 + t88 - 0.2e1 * t16 * t61;
		t390 = -0.2e1 * t206 + 0.2e1 * t42 - 0.6e1 * t19 - 0.2e1 * t74 * t160 + 0.3e1 * t16 * t125 - 0.2e1 * t74 * t32 - 0.4e1 * t74 * t342 + 0.3e1 * t15 * t149 + 0.2e1 * t41 - t16 * t128 - 0.4e1 * t170 * t128 - t15 * t120 + t78;
		t411 = -t16 * t129 - 0.2e1 * t16 * t42 - 0.4e1 * t16 * t299 - t16 * t73 + t15 * t108 * t5 - 0.3e1 * t16 * t210 + 0.4e1 * t1 * t42 + t16 * t50 + t15 * t146 - t15 * t153 - 0.2e1 * t31 * t199 - 0.2e1 * t16 * t113;
		t430 = -t31 * t75 - t16 * t71 - t144 * t7 + 0.3e1 * t15 * t163 + t16 * t47 - t56 * t5 + 0.8e1 * t25 * t140 - t98 * t162 * t5 - 0.3e1 * t27 * t7 + t1 * t172 + t4 * t128 + 0.2e1 * t40 * t129 + 0.2e1 * t22;
		t441 = n * n;
		t444 = M_PI * M_PI;
		t449 = pow(t2 + 0.1e1 + t8, 0.2e1);
		t453 = pow(t2 - 0.1e1 - t8, 0.2e1);

		u2a = (t2 + t6 - t7 - t9) * deltaRho * t13 * (t63 + t93 + t123 + t156 + t191 + t217 + t244 + t264 + t286 + t307 + t330 + t349 + t371 + t390 + t411 + t430) / t7 / (0.1e1 + t5) / t441 / n / t444 / M_PI / t449 / t453 / t37 / viscosity / (0.1e1 + t17) / 0.2e1;

		t1 = n * M_PI;
		t3 = sin(t1 * dx);
		t5 = 0.1e1 / cosh(t1);
		t6 = t5 * t5;
		t7 = t6 * t6;
		t8 = startBlockY * n;
		t9 = t8 * M_PI;
		t10 = 0.1e1 / cosh(t9);
		t11 = t10 * t10;
		t13 = y * n;
		t14 = t13 * M_PI;
		t15 = tanh(t14);
		t16 = t15 * n;
		t17 = t16 * M_PI;
		t18 = t6 * t5;
		t19 = tanh(t1);
		t20 = t18 * t19;
		t21 = tanh(t9);
		t22 = t10 * t21;
		t23 = t20 * t22;
		t26 = 0.1e1 / cosh(t14);
		t27 = t26 * t26;
		t28 = t27 * t11;
		t29 = t28 * t6;
		t30 = t15 * t15;
		t31 = t30 * t5;
		t32 = t19 * t19;
		t33 = t32 * t10;
		t34 = t31 * t33;
		t37 = t27 * n;
		t38 = t37 * M_PI;
		t39 = t32 * t19;
		t40 = t5 * t39;
		t41 = t40 * t10;
		t44 = t20 * t10;
		t47 = t21 * t21;
		t48 = t27 * t47;
		t49 = t48 * t6;
		t50 = t6 * t19;
		t52 = t15 * t6;
		t53 = t52 * t19;
		t56 = t30 * startBlockY;
		t57 = t56 * n;
		t58 = M_PI * t19;
		t59 = t21 * t6;
		t60 = t58 * t59;
		t64 = t19 * t10;
		t65 = t64 * t21;
		t68 = t27 * t18;
		t71 = t68 * t22;
		t74 = -t7 * t11 - 0.4e1 * t17 * t23 - t29 + 0.6e1 * t14 * t34 - 0.2e1 * t38 * t41 - 0.2e1 * t1 * t44 - t49 + 0.2e1 * t50 - 0.4e1 * t14 * t53 - 0.4e1 * t57 * t60 - 0.2e1 * t1 * t18 * t65 + 0.2e1 * t68 * t65 + 0.2e1 * t14 * t71;
		t75 = t30 * t18;
		t76 = t75 * t22;
		t78 = startBlockY * t6;
		t79 = t78 * t19;
		t84 = t15 * t18;
		t85 = t84 * t22;
		t87 = t27 * startBlockY;
		t91 = M_PI * t32;
		t92 = t91 * t59;
		t95 = t15 * t5;
		t96 = t39 * t10;
		t97 = t95 * t96;
		t99 = t95 * t64;
		t101 = M_PI * t6;
		t104 = t75 * t64;
		t106 = t30 * t11;
		t107 = t106 * t50;
		t109 = t15 * startBlockY;
		t110 = t109 * n;
		t111 = M_PI * t11;
		t112 = t6 * t32;
		t113 = t111 * t112;
		t116 = M_PI * t27;
		t117 = t13 * t116;
		t118 = t87 * n;
		t121 = 0.2e1 * t76 - 0.4e1 * t17 * t79 - 0.2e1 * t7 * t21 + 0.4e1 * t85 - 0.6e1 * t87 * t1 * t32 - 0.2e1 * t57 * t92 - 0.4e1 * t97 - 0.12e2 * t99 + 0.4e1 * t16 * t101 + 0.2e1 * t104 + 0.2e1 * t107 + 0.2e1 * t110 * t113 + t117 + 0.4e1 * t118 * t60;
		t123 = t11 * t6;
		t126 = t5 * t19;
		t127 = t126 * t22;
		t130 = M_PI * t7;
		t132 = t18 * t10;
		t135 = t27 * t5;
		t136 = t135 * t10;
		t138 = t68 * t64;
		t143 = t130 * t47;
		t145 = t30 * t7;
		t146 = t145 * t11;
		t150 = t5 * t32;
		t151 = t150 * t22;
		t155 = 0.2e1 * t1 * t123 - 0.6e1 * t127 - t13 * t101 + t8 * t130 - 0.2e1 * t1 * t132 - 0.2e1 * t136 + 0.2e1 * t138 + 0.6e1 * t118 * t91 * t11 + t57 * t143 + t14 * t146 - 0.4e1 * t17 * t41 - 0.6e1 * t151 + 0.4e1 * t1 * t59;
		t156 = t47 * t6;
		t159 = t95 * t10;
		t161 = M_PI * t47;
		t162 = t161 * t112;
		t165 = t40 * t22;
		t175 = t130 * t11;
		t177 = t91 * t6;
		t183 = 0.2e1 * t1 * t156 - 0.4e1 * t159 - 0.2e1 * t110 * t162 - 0.2e1 * t38 * t165 - 0.2e1 * t165 - t7 + 0.2e1 * t110 * t143 + 0.2e1 * t118 * t92 + 0.2e1 * t38 * t79 + t13 * t175 - t57 * t177 + t118 * t177 - t14 * t49 - 0.12e2 * t95 * t65;
		t186 = t59 * t19;
		t189 = t32 * t32;
		t190 = M_PI * t189;
		t193 = t95 * t22;
		t199 = t5 * t10;
		t202 = t30 * t21;
		t203 = t202 * t6;
		t206 = t135 * t96;
		t209 = t135 * t22;
		t212 = t95 * t33;
		t214 = t106 * t112;
		t218 = t31 * t64;
		t220 = 0.16e2 * t17 * t186 + t118 * t190 * t11 + 0.4e1 * t14 * t193 - 0.6e1 * t31 * t65 + 0.2e1 * t23 - 0.2e1 * t1 * t199 - 0.2e1 * t14 * t203 - 0.2e1 * t14 * t206 - 0.2e1 * t14 * t209 - 0.12e2 * t212 + t214 + 0.2e1 * t13 * t116 * t21 - 0.6e1 * t218;
		t226 = t202 * t50;
		t229 = t30 * t47;
		t230 = t229 * t50;
		t233 = t68 * t10;
		t236 = t31 * t96;
		t241 = t111 * t50;
		t244 = t101 * t11;
		t247 = t123 * t32;
		t250 = t156 * t19;
		t253 = t57 * t113 + t28 + 0.12e2 * t14 * t99 + 0.2e1 * t37 * t101 - 0.4e1 * t14 * t226 + 0.2e1 * t230 + 0.4e1 * t226 + 0.2e1 * t14 * t233 + 0.2e1 * t14 * t236 + 0.4e1 * t1 * t50 + 0.4e1 * t110 * t241 + 0.4e1 * t16 * t244 + 0.4e1 * t17 * t247 - 0.2e1 * t14 * t250;
		t256 = t161 * t50;
		t260 = t101 * t21;
		t263 = t123 * t19;
		t268 = t31 * t22;
		t272 = t229 * t112;
		t273 = t84 * t64;
		t275 = M_PI * t30;
		t276 = t13 * t275;
		t280 = t15 * t21;
		t281 = t280 * t50;
		t283 = t118 * t162 - 0.4e1 * t110 * t256 - t118 * t113 + 0.4e1 * t37 * t260 - 0.2e1 * t14 * t263 - 0.4e1 * t14 * t85 - 0.2e1 * t268 - 0.2e1 * t14 * t136 + t272 + 0.4e1 * t273 + 0.2e1 * t276 * t165 - 0.4e1 * t193 + 0.8e1 * t281;
		t285 = t30 * t6;
		t286 = t285 * t32;
		t288 = t156 * t32;
		t291 = t101 * t19;
		t294 = t48 * t50;
		t296 = t30 * n;
		t297 = t296 * M_PI;
		t298 = t132 * t21;
		t301 = t28 * t50;
		t303 = t48 * t112;
		t305 = t28 * t112;
		t310 = -t8 * t175 - t14 * t286 + 0.2e1 * t1 * t288 + 0.8e1 * t16 * t291 - 0.2e1 * t294 + t156 - 0.2e1 * t297 * t298 - 0.2e1 * t301 - t303 - 0.2e1 * t199 - t305 + 0.2e1 * t71 - 0.2e1 * t276 * t23 - t9 * t288;
		t314 = t59 * t32;
		t321 = t135 * t33;
		t326 = t33 * t21;
		t333 = t27 * t21;
		t334 = t333 * t50;
		t338 = t15 * t11;
		t339 = t338 * t50;
		t341 = t27 * t189;
		t342 = 0.4e1 * t1 * t314 - 0.2e1 * t87 * t1 * t21 - t7 * t47 - 0.6e1 * t321 - 0.4e1 * t87 * t1 * t39 - 0.6e1 * t135 * t326 - t14 * t247 + 0.2e1 * t14 * t165 - 0.2e1 * t236 - 0.4e1 * t334 + 0.2e1 * t57 * t241 + 0.4e1 * t339 + t341;
		t343 = t15 * t47;
		t344 = t343 * t112;
		t346 = t343 * t50;
		t348 = M_PI * t15;
		t349 = t13 * t348;
		t365 = t161 * t6;
		t372 = M_PI * startBlockY * t6;
		t374 = 0.2e1 * t344 + 0.4e1 * t346 + 0.12e2 * t349 * t151 - 0.2e1 * t14 * t44 - 0.2e1 * t57 * t256 + 0.4e1 * t1 * t250 + 0.8e1 * t1 * t186 + 0.8e1 * t17 * t263 - 0.2e1 * t57 * t260 - 0.6e1 * t34 - t13 * t365 - 0.2e1 * t1 * t41 - 0.2e1 * t14 * t107 - t296 * t372;
		t379 = t150 * t10;
		t388 = t341 * t47;
		t390 = t96 * t21;
		t394 = M_PI * t18 * t10;
		t402 = -0.2e1 * t14 * t23 + t57 * t244 - 0.6e1 * t297 * t379 + 0.2e1 * t296 * t244 - 0.2e1 * t297 * t23 - t1 * t78 + 0.2e1 * t59 + t14 * t388 - 0.2e1 * t135 * t390 - 0.2e1 * t296 * t394 - 0.2e1 * t110 * t175 - t87 * t1 - 0.2e1 * t297 * t44;
		t403 = t27 * t39;
		t404 = t403 * t47;
		t409 = t15 * t7;
		t410 = t409 * t47;
		t414 = t333 * t112;
		t425 = t130 * t21;
		t429 = t341 * t11;
		t431 = 0.4e1 * t14 * t404 + 0.2e1 * t296 * t365 - 0.2e1 * t410 - 0.2e1 * t16 * t372 - 0.2e1 * t414 - 0.4e1 * t17 * t165 + t112 + t8 * t143 - 0.6e1 * t38 * t379 - t14 * t214 + 0.4e1 * t13 * t116 * t19 + 0.2e1 * t8 * t425 + 0.2e1 * t132 + t14 * t429;
		t447 = t126 * t10;
		t450 = t199 * t21;
		t455 = M_PI * t5 * t10;
		t460 = t1 * t5;
		t463 = -0.6e1 * t118 * t91 * t47 - 0.6e1 * t38 * t151 - 0.2e1 * t297 * t41 - 0.4e1 * t17 * t298 - 0.6e1 * t297 * t151 + t48 - 0.6e1 * t135 * t65 - 0.6e1 * t38 * t447 - 0.2e1 * t1 * t450 + t9 * t247 - 0.2e1 * t37 * t455 - 0.6e1 * t1 * t379 - 0.6e1 * t460 * t326;
		t464 = t135 * t64;
		t467 = t403 * t21;
		t491 = -0.6e1 * t464 + t37 * t372 + 0.8e1 * t467 - 0.6e1 * t1 * t447 - 0.4e1 * t14 * t273 + 0.2e1 * t296 * t177 + 0.2e1 * t37 * t244 - 0.6e1 * t460 * t65 + 0.6e1 * t14 * t151 + 0.8e1 * t16 * t260 - 0.2e1 * t460 * t390 + 0.4e1 * t296 * t291 - 0.12e2 * t95 * t326 - 0.2e1 * t37 * t394;
		t501 = t202 * t112;
		t512 = t1 * t7;
		t518 = t27 * t19;
		t519 = t518 * t11;
		t522 = -0.4e1 * t16 * t394 - 0.4e1 * t110 * t260 - 0.4e1 * t95 * t390 - 0.2e1 * t1 * t79 - 0.2e1 * t14 * t501 + 0.2e1 * t297 * t288 - 0.2e1 * t14 * t314 + 0.4e1 * t84 * t65 + 0.6e1 * t276 * t127 + t56 * t512 - 0.2e1 * t38 * t298 + 0.2e1 * t37 * t365 + 0.4e1 * t14 * t519;
		t540 = t84 * t10;
		t550 = -0.6e1 * t14 * t464 - t8 * t177 + 0.2e1 * t110 * t244 - 0.4e1 * t14 * t339 - 0.2e1 * t1 * t298 + 0.12e2 * t14 * t212 - 0.2e1 * t38 * t44 - t13 * t177 - 0.4e1 * t87 * t1 * t19 + 0.4e1 * t540 - 0.2e1 * t297 * t165 + 0.2e1 * t296 * t101 - 0.2e1 * t13 * t394 - 0.2e1 * t9 * t314;
		t557 = t338 * t112;
		t567 = t518 * t21;
		t582 = -0.4e1 * t17 * t450 - 0.2e1 * t14 * t557 + 0.4e1 * t110 * t425 + 0.2e1 * t14 * t268 + 0.2e1 * t38 * t247 + t13 * t143 + 0.8e1 * t14 * t567 - 0.4e1 * t9 * t186 + 0.2e1 * t13 * t348 * t7 - 0.2e1 * t209 + 0.4e1 * t14 * t97 + 0.2e1 * t57 * t425 - 0.12e2 * t17 * t379;
		t587 = t145 * t47;
		t598 = t145 * t21;
		t609 = -t57 * t162 - 0.4e1 * t14 * t334 - t57 * t175 + t14 * t587 - 0.4e1 * t14 * t540 - 0.12e2 * t17 * t151 - t146 + 0.2e1 * t14 * t410 - 0.12e2 * t118 * t91 * t21 + 0.2e1 * t14 * t598 + 0.4e1 * t38 * t263 - 0.2e1 * t297 * t450 + 0.4e1 * t14 * t159 + 0.2e1 * t38 * t288;
		t611 = t52 * t32;
		t614 = t27 * t32;
		t615 = t614 * t21;
		t618 = M_PI * t39;
		t634 = t285 * t19;
		t637 = t280 * t112;
		t639 = t27 * t6;
		t640 = t639 * t32;
		t642 = t280 * t6;
		t645 = -0.2e1 * t14 * t611 + 0.12e2 * t14 * t615 + 0.4e1 * t118 * t618 * t11 - 0.2e1 * t118 * t241 - t145 + 0.4e1 * t118 * t58 * t11 + 0.2e1 * t118 * t256 - 0.2e1 * t14 * t414 + 0.4e1 * t13 * t116 * t39 - 0.2e1 * t14 * t634 + 0.4e1 * t637 - t14 * t640 - 0.4e1 * t14 * t642;
		t654 = t333 * t6;
		t673 = -0.2e1 * t13 * t260 - t13 * t275 * t6 + 0.4e1 * t296 * t260 - 0.6e1 * t117 * t151 - 0.2e1 * t14 * t654 + 0.6e1 * t14 * t218 + 0.4e1 * t349 * t165 + 0.4e1 * t38 * t314 - 0.12e2 * t17 * t447 - 0.12e2 * t17 * t127 + 0.2e1 * t13 * t455 + t27 - 0.4e1 * t349 * t23 - 0.6e1 * t31 * t326;
		t682 = t409 * t11;
		t702 = t13 * t275 * t7 - 0.2e1 * t117 * t165 + 0.6e1 * t276 * t151 - 0.2e1 * t682 - 0.8e1 * t14 * t281 + 0.2e1 * t297 * t247 - 0.2e1 * t110 * t365 - 0.2e1 * t14 * t230 - 0.4e1 * t14 * t637 + 0.4e1 * t38 * t250 - t87 * t1 * t47 + 0.2e1 * t117 * t23 - 0.2e1 * t8 * t260;
		t705 = t518 * t47;
		t716 = t343 * t6;
		t719 = t614 * t11;
		t721 = t403 * t11;
		t726 = t409 * t21;
		t728 = t338 * t6;
		t731 = 0.2e1 * t13 * t425 + 0.4e1 * t14 * t705 - t13 * t116 * t6 - 0.8e1 * t110 * t60 - 0.4e1 * t16 * t455 - 0.2e1 * t14 * t76 - 0.2e1 * t14 * t716 + 0.6e1 * t719 + 0.4e1 * t721 + 0.4e1 * t297 * t314 + 0.4e1 * t642 - 0.4e1 * t726 + 0.2e1 * t728 + 0.2e1 * t716;
		t734 = t106 * t6;
		t735 = t229 * t6;
		t740 = t614 * t47;
		t742 = t31 * t10;
		t744 = 0.2e1 * t203 + t734 + t735 + t247 + 0.2e1 * t263 - 0.2e1 * t598 - 0.2e1 * t450 - t587 - t14 * t272 + 0.6e1 * t740 + t429 - 0.2e1 * t742 + t286;
		t747 = t75 * t10;
		t753 = t341 * t21;
		t761 = -0.2e1 * t41 - 0.6e1 * t379 + 0.2e1 * t747 + 0.4e1 * t186 + 0.2e1 * t314 + 0.2e1 * t611 - 0.2e1 * t654 + 0.2e1 * t753 - t14 * t735 + 0.2e1 * t250 + t288 + 0.2e1 * t1 * t6 + 0.4e1 * t705 + 0.4e1 * t519;
		t771 = t639 * t19;
		t779 = 0.2e1 * t233 + 0.12e2 * t615 + 0.8e1 * t567 + 0.6e1 * t14 * t127 + 0.2e1 * t634 - 0.2e1 * t771 + 0.4e1 * t53 + 0.4e1 * t404 + t388 + 0.2e1 * t44 + 0.8e1 * t38 * t186 + 0.2e1 * t298 + t123;
		t801 = -0.4e1 * t14 * t346 - 0.6e1 * t447 - 0.6e1 * t38 * t127 + 0.6e1 * t14 * t740 - t14 * t29 + 0.2e1 * t1 * t247 - t14 * t734 - 0.4e1 * t17 * t44 - t640 + t8 * t244 - 0.2e1 * t14 * t344 - 0.4e1 * t110 * t92 + 0.2e1 * t557 - 0.2e1 * t31 * t390;
		t830 = -0.4e1 * t118 * t58 * t47 - 0.4e1 * t14 * t186 + 0.2e1 * t9 * t263 - 0.2e1 * t9 * t250 + 0.2e1 * t14 * t138 - 0.2e1 * t14 * t104 + 0.4e1 * t17 * t288 + 0.2e1 * t501 + 0.6e1 * t13 * t116 * t32 + 0.2e1 * t37 * t177 + 0.6e1 * t14 * t379 + 0.2e1 * t14 * t41 - t118 * t190 * t47;
		t854 = -t87 * t1 * t189 - 0.6e1 * t14 * t321 + 0.2e1 * t333 + t13 * t130 + 0.2e1 * t75 * t65 + 0.8e1 * t17 * t314 + 0.2e1 * t109 * t512 + 0.4e1 * t297 * t250 - t8 * t365 + 0.4e1 * t14 * t721 + t6 + 0.4e1 * t16 * t365 - 0.6e1 * t297 * t447 - 0.2e1 * t14 * t301;
		t883 = 0.4e1 * t37 * t291 - 0.2e1 * t296 * t455 + 0.8e1 * t17 * t250 - 0.2e1 * t118 * t190 * t21 + t87 * t1 * t11 + 0.12e2 * t349 * t127 + 0.6e1 * t14 * t719 + 0.2e1 * t52 + 0.2e1 * t14 * t450 - 0.2e1 * t38 * t23 + 0.4e1 * t297 * t263 + 0.2e1 * t1 * t112 - 0.2e1 * t297 * t79;
		t910 = -0.2e1 * t13 * t291 - 0.6e1 * t117 * t127 + t285 - 0.2e1 * t13 * t348 * t6 + 0.8e1 * t297 * t186 - t13 * t244 - t57 * t365 - 0.4e1 * t118 * t618 * t47 + 0.2e1 * t14 * t753 + t13 * t116 * t11 + 0.2e1 * t118 * t260 + 0.6e1 * t14 * t447 - 0.2e1 * t206 - 0.8e1 * t118 * t618 * t21;
		t936 = -0.2e1 * t14 * t771 + 0.2e1 * t14 * t742 + t13 * t116 * t47 - 0.6e1 * t297 * t127 - 0.2e1 * t14 * t728 - t14 * t288 - 0.2e1 * t38 * t450 + 0.6e1 * t614 - 0.2e1 * t14 * t294 + 0.4e1 * t518 - 0.2e1 * t110 * t177 - 0.2e1 * t14 * t747 - 0.8e1 * t118 * t58 * t21;
		t957 = 0.2e1 * t14 * t682 + 0.4e1 * t14 * t726 + 0.4e1 * t16 * t177 + t13 * t116 * t189 + 0.4e1 * t403 - 0.2e1 * t14 * t298 - t14 * t303 - t118 * t244 - t639 + 0.4e1 * t1 * t263 - 0.2e1 * t409 + t118 * t365 - t14 * t305 + 0.8e1 * t14 * t467;
		t969 = pow(t5 + 0.1e1 + t19, 0.2e1);
		t974 = pow(t5 - 0.1e1 - t19, 0.2e1);
		t978 = n * n;
		t982 = M_PI * M_PI;

		u2b = deltaRho * t3 * (t910 + t74 + t582 + t310 + t830 + t491 + t883 + t645 + t342 + t957 + t283 + t673 + t220 + t121 + t183 + t609 + t550 + t731 + t374 + t702 + t402 + t431 + t854 + t522 + t761 + t744 + t779 + t463 + t155 + t253 + t936 + t801) / t10 / (0.1e1 + t21) / t969 / t974 / t26 / t978 / n / t982 / M_PI / viscosity / (0.1e1 + t15) / 0.2e1;

		if ( y > startBlockY ) {
			u1=u1b; u2=u2b;
		}
		else {
			u1 = u1a; u2=u2a;
		}

		u1 *= cos(n*M_PI*x);
		velocity[ J_AXIS ] += u1;
		u2 *= sin(n*M_PI*x);
		velocity[ I_AXIS ] += u2;
	}
}


void _AnalyticBlock_PressureFunction( void* analyticSolution, FeVariable* analyticFeVariable, double* coord, double* pressure ) {
	AnalyticBlock*          self           = (AnalyticBlock*)analyticSolution;
	double                  n;
	double                  x, y;
	double                  viscosity      = 1.0;
	double                  deltaRho       = 1.0;
	double                  startBlockX    = self->startBlockX;
	double                  endBlockX      = self->endBlockX;
	double                  startBlockY    = self->startBlockY;
	double                  x0;
	double                  dx;
	double                  u2, u3, u5;
	double                  u2a, u2b;
	double                  u3a, u3b;
	double t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13,t14,t15,t16,t17,t18,t19,t20,t21,t22,t23,t24,t25,t26,t27,t28,t29,t30,t31,t32,t33,t34,t35,t36,t37,t38,t39,t40;
	double t41,t42,t44,t45,t46,t47,t48,t49,t50,t51,t52,t53,t54,t55,t56,t57,t58,t59,t60,t61,t62,t63,t64,t65,t66,t67,t68,t69,t70,t71,t72,t73,t74,t75,t76,t77,t78,t79;
	double t82,t83,t84,t85,t86,t87,t88,t89,t90,t91,t92,t93,t94,t95,t96,t97,t98,t99,t100,t101,t102,t103,t104,t105,t106,t107,t108,t109,t110,t111,t112,t113,t114,t115,t116,t117,t118,t119,t120;
	double t121,t123,t124,t125,t126,t127,t128,t129,t130,t131,t132,t133,t134,t135,t136,t138,t139,t140,t141,t143,t144,t145,t146,t148,t149,t150,t151,t153,t154,t155,t156,t157,t159,t160;
	double t161,t162,t163,t164,t165,t166,t167,t169,t170,t171,t172,t173,t174,t175,t177,t178,t179,t180,t182,t183,t184,t186,t188,t189,t190,t191,t192,t193,t194,t196,t197,t199;
	double t201,t202,t203,t205,t206,t208,t209,t210,t211,t212,t214,t216,t217,t218,t219,t220,t222,t225,t226,t228,t229,t230,t231,t233,t234,t235,t236,t238,t239;
	double t241,t244,t247,t248,t249,t250,t252,t253,t255,t256,t257,t258,t259,t260,t262,t263,t264,t265,t266,t267,t268,t269,t270,t272,t273,t275,t276,t277,t279,t280;
	double t281,t282,t283,t284,t285,t286,t287,t288,t289,t291,t294,t295,t296,t297,t298,t299,t300,t301,t303,t304,t305,t307,t308,t309,t310,t313,t314,t317;
	double t321,t324,t326,t328,t329,t330,t331,t333,t334,t336,t338,t339,t341,t342,t343,t344,t346,t347,t348,t349,t350,t352,t353,t356,t359;
	double t362,t365,t366,t367,t369,t370,t371,t372,t374,t376,t378,t379,t380,t382,t385,t387,t388,t389,t390,t392,t393,t394,t396,t399;
	double t402,t403,t404,t405,t408,t409,t410,t411,t412,t414,t415,t416,t421,t422,t423,t425,t428,t429,t430,t431;
	double t441,t442,t444,t447,t449,t450,t452,t453,t455,t460,t461,t463,t464,t467,t471,t473,t474,t476,t479;
	double t483,t484,t485,t491,t492,t500,t501,t511,t512,t517,t518,t519;
	double t522,t529,t536,t537,t540,t544,t550,t551,t553,t556,t557;
	double t567,t573,t580,t582,t587,t598,t600;
	double t605,t609,t611,t614,t615,t618,t626,t634,t637,t638,t639,t640;
	double t642,t645,t646,t651,t652,t654,t657,t659,t662,t664,t673;
	double t682,t699,t702,t705,t716,t719;
	double t721,t726,t727,t728,t730,t731,t734,t735,t740,t742,t744,t747,t753,t755,t756;
	double t761,t771,t779,t783,t795;
	double t801,t806,t809,t811,t816,t823,t828,t830;
	double t849,t854,t877;
	double t883,t904,t910;
	double t933,t936,t957;
	double t962,t969,t974,t978,t982,t988;
	double t1001,t1004,t1011,t1015;

	
	/* Find coordinate of node */
	x = coord[ I_AXIS ];
	y = coord[ J_AXIS ];

	x0 = 0.0;
	dx = endBlockX - startBlockX;

	/* Initialise */
	*pressure = 0.0;

	for( n = 1.0 ; n < MAX_FOURIER_TERMS ; n++ ) {


		t1 = n * M_PI;
		t2 = 0.1e1 / cosh(t1);
		t3 = startBlockY * n;
		t4 = t3 * M_PI;
		t5 = tanh(t4);
		t6 = t2 * t5;
		t7 = 0.1e1 / cosh(t4);
		t8 = tanh(t1);
		t9 = t8 * t7;
		t13 = sin(t1 * dx);
		t15 = y * n;
		t16 = t15 * M_PI;
		t17 = tanh(t16);
		t19 = t17 * t8 * t7;
		t22 = t17 * t2;
		t25 = t17 * n;
		t26 = M_PI * t2;
		t27 = t8 * t8;
		t28 = t26 * t27;
		t31 = t2 * startBlockY;
		t32 = t1 * t5;
		t34 = t17 * t17;
		t35 = t34 * t2;
		t37 = 0.1e1 / cosh(t16);
		t38 = t37 * t37;
		t39 = t38 * n;
		t40 = t39 * M_PI;
		t41 = t2 * t8;
		t42 = t41 * t5;
		t45 = t38 * t2;
		t46 = t27 * t5;
		t47 = t45 * t46;
		t48 = t2 * t2;
		t49 = t38 * t48;
		t50 = t49 * t9;
		t53 = t8 * t5;
		t54 = t22 * t53;
		t56 = t48 * t2;
		t57 = t17 * t56;
		t60 = t17 * t48;
		t61 = t60 * t9;
		t63 = 0.6e1 * t16 * t19 - 0.2e1 * t22 * t4 + 0.4e1 * t25 * t28 - t31 * t32 - t35 * t4 + 0.4e1 * t40 * t42 + t47 + t50 + 0.2e1 * t39 * t26 + 0.4e1 * t54 + 0.2e1 * t57 * t4 + 0.2e1 * t61;
		t64 = t34 * n;
		t67 = t34 * t56;
		t69 = t35 * t53;
		t71 = t35 * t46;
		t72 = t34 * t48;
		t73 = t72 * t9;
		t74 = t22 * startBlockY;
		t75 = t1 * t27;
		t78 = t48 * t7;
		t83 = M_PI * t7;
		t85 = t56 * startBlockY;
		t88 = t2 * t27;
		t93 = 0.2e1 * t64 * t26 + t67 * t4 + 0.2e1 * t69 + t71 + t73 - 0.2e1 * t74 * t75 - 0.2e1 * t1 * t78 + 0.2e1 * t1 * t6 - t3 * t83 + t85 * t1 - t31 * t1 + 0.2e1 * t1 * t88 + 0.4e1 * t1 * t41;
		t98 = t31 * n;
		t99 = t8 * M_PI;
		t104 = t17 * startBlockY;
		t105 = t1 * t7;
		t108 = M_PI * t56;
		t110 = t45 * t53;
		t113 = t22 * t46;
		t117 = t38 * t56;
		t118 = t117 * t5;
		t120 = t26 * t5;
		t123 = 0.4e1 * t25 * t26 - t15 * t26 - 0.2e1 * t98 * t99 * t5 + t15 * t83 - 0.2e1 * t104 * t105 + t15 * t108 + 0.2e1 * t110 + t85 * t32 + 0.2e1 * t113 + 0.2e1 * t16 * t110 - t16 * t118 + 0.4e1 * t25 * t120;
		t125 = t34 * t27 * t7;
		t128 = t48 * t8 * t7;
		t129 = t88 * t5;
		t130 = t45 * t27;
		t131 = t67 * t5;
		t133 = t49 * t7;
		t135 = M_PI * t38;
		t140 = t26 * t8;
		t143 = t104 * n;
		t144 = t27 * t8;
		t146 = M_PI * t144 * t7;
		t149 = t99 * t7;
		t153 = M_PI * t48 * t7;
		t156 = -0.3e1 * t125 + t128 + t129 + t130 - t131 - t15 * t28 + t16 * t133 - t15 * t135 * t7 + t15 * t135 * t2 - 0.2e1 * t15 * t140 - 0.2e1 * t143 * t146 - 0.6e1 * t143 * t149 - 0.2e1 * t39 * t153;
		t159 = t45 * startBlockY;
		t160 = t1 * t46;
		t162 = M_PI * t27;
		t163 = t162 * t7;
		t166 = t38 * startBlockY;
		t170 = M_PI * t25;
		t171 = startBlockY * t48;
		t172 = t171 * t7;
		t177 = t35 * t8;
		t182 = t171 * t9;
		t186 = t34 * t8 * t7;
		t191 = -t159 * t160 - 0.6e1 * t143 * t163 - t166 * t105 + 0.2e1 * t39 * t120 + 0.2e1 * t170 * t172 + 0.2e1 * t1 * t129 - 0.2e1 * t16 * t177 - 0.2e1 * t40 * t128 + 0.2e1 * t170 * t182 + 0.3e1 * t16 * t186 + t117 * t4 - t45 * t4;
		t192 = t35 * t5;
		t194 = t166 * n;
		t199 = t1 * t8;
		t206 = t57 * t5;
		t210 = t38 * t27 * t7;
		t217 = -t16 * t192 - t194 * t146 + 0.2e1 * t57 * startBlockY * t32 - 0.4e1 * t74 * t199 - t56 - 0.4e1 * t16 * t54 - 0.3e1 * t9 - 0.2e1 * t57 + 0.2e1 * t16 * t206 - 0.3e1 * t210 + 0.2e1 * t1 * t2 - 0.4e1 * t25 * t153 + t16 * t131;
		t225 = M_PI * t17;
		t236 = t38 * t8 * t7;
		t239 = t45 * t5;
		t241 = t45 * t8;
		t244 = -0.2e1 * t16 * t69 - 0.2e1 * t159 * t199 - 0.2e1 * t1 * t128 + 0.2e1 * t15 * t225 * t56 + t35 - 0.2e1 * t17 * t7 + 0.2e1 * t15 * t225 * t7 - t159 * t75 - 0.3e1 * t16 * t236 - t117 + t16 * t239 + 0.2e1 * t16 * t241;
		t249 = t64 * M_PI;
		t253 = t38 * t144 * t7;
		t258 = t22 * t5;
		t264 = -t38 * t7 + t6 - 0.3e1 * t194 * t163 + t40 * t172 + t249 * t172 - t3 * t146 - t16 * t253 - t15 * t135 * t56 - t159 * t32 + t2 - 0.2e1 * t16 * t258 + t249 * t182 + t67 * startBlockY * t32;
		t270 = t60 * t7;
		t275 = t17 * t144 * t7;
		t279 = t72 * t7;
		t282 = t34 * t144 * t7;
		t284 = t35 * t27;
		t286 = 0.4e1 * t64 * t140 - 0.2e1 * t16 * t270 + t16 * t130 + 0.2e1 * t16 * t275 + t40 * t182 - t16 * t279 + t16 * t282 + t284 - 0.3e1 * t186 + t192 + t279 - t282;
		t287 = t35 * startBlockY;
		t295 = t22 * t27;
		t299 = t22 * t8;
		t304 = t17 * t27 * t7;
		t307 = -0.2e1 * t287 * t199 + 0.2e1 * t249 * t129 - t34 * t7 + 0.4e1 * t249 * t42 + 0.2e1 * t295 + 0.2e1 * t241 - 0.3e1 * t236 + t239 - t253 + 0.4e1 * t299 - t118 - 0.2e1 * t64 * t153 + 0.6e1 * t16 * t304;
		t309 = t34 * startBlockY;
		t317 = M_PI * t34;
		t328 = t309 * n;
		t330 = -t309 * t105 + t117 * startBlockY * t32 - t7 + 0.4e1 * t39 * t140 + 0.2e1 * t64 * t120 + t45 + t15 * t317 * t7 - 0.2e1 * t249 * t128 - 0.3e1 * t3 * t149 + 0.2e1 * t39 * t28 - 0.3e1 * t194 * t149 - t328 * t146;
		t342 = t1 * t53;
		t349 = -0.3e1 * t328 * t149 + t15 * t317 * t56 - t67 - t16 * t284 - 0.3e1 * t328 * t163 + 0.2e1 * t64 * t28 - t15 * t317 * t2 - 0.2e1 * t159 * t342 - t287 * t160 + t133 + 0.2e1 * t258 + 0.2e1 * t270 - 0.2e1 * t275;
		t371 = -0.6e1 * t304 - t287 * t75 + 0.2e1 * t177 + 0.4e1 * t170 * t129 - 0.2e1 * t15 * t225 * t2 - t287 * t32 - 0.3e1 * t3 * t163 - 0.2e1 * t287 * t342 - 0.2e1 * t16 * t295 + 0.8e1 * t170 * t42 + t88 - 0.2e1 * t16 * t61;
		t390 = -0.2e1 * t206 + 0.2e1 * t42 - 0.6e1 * t19 - 0.2e1 * t74 * t160 + 0.3e1 * t16 * t125 - 0.2e1 * t74 * t32 - 0.4e1 * t74 * t342 + 0.3e1 * t15 * t149 + 0.2e1 * t41 - t16 * t128 - 0.4e1 * t170 * t128 - t15 * t120 + t78;
		t411 = -t16 * t129 - 0.2e1 * t16 * t42 - 0.4e1 * t16 * t299 - t16 * t73 + t15 * t108 * t5 - 0.3e1 * t16 * t210 + 0.4e1 * t1 * t42 + t16 * t50 + t15 * t146 - t15 * t153 - 0.2e1 * t31 * t199 - 0.2e1 * t16 * t113;
		t430 = -t31 * t75 - t16 * t71 - t144 * t7 + 0.3e1 * t15 * t163 + t16 * t47 - t56 * t5 + 0.8e1 * t25 * t140 - t98 * t162 * t5 - 0.3e1 * t27 * t7 + t1 * t172 + t4 * t128 + 0.2e1 * t40 * t129 + 0.2e1 * t22;
		t441 = n * n;
		t444 = M_PI * M_PI;
		t449 = pow(t2 + 0.1e1 + t8, 0.2e1);
		t453 = pow(t2 - 0.1e1 - t8, 0.2e1);

		u2a = (t2 + t6 - t7 - t9) * deltaRho * t13 * (t63 + t93 + t123 + t156 + t191 + t217 + t244 + t264 + t286 + t307 + t330 + t349 + t371 + t390 + t411 + t430) / t7 / (0.1e1 + t5) / t441 / n / t444 / M_PI / t449 / t453 / t37 / viscosity / (0.1e1 + t17) / 0.2e1;

		t1 = n * M_PI;
		t3 = sin(t1 * dx);
		t5 = 0.1e1 / cosh(t1);
		t6 = t5 * t5;
		t7 = t6 * t6;
		t8 = startBlockY * n;
		t9 = t8 * M_PI;
		t10 = 0.1e1 / cosh(t9);
		t11 = t10 * t10;
		t13 = y * n;
		t14 = t13 * M_PI;
		t15 = tanh(t14);
		t16 = t15 * n;
		t17 = t16 * M_PI;
		t18 = t6 * t5;
		t19 = tanh(t1);
		t20 = t18 * t19;
		t21 = tanh(t9);
		t22 = t10 * t21;
		t23 = t20 * t22;
		t26 = 0.1e1 / cosh(t14);
		t27 = t26 * t26;
		t28 = t27 * t11;
		t29 = t28 * t6;
		t30 = t15 * t15;
		t31 = t30 * t5;
		t32 = t19 * t19;
		t33 = t32 * t10;
		t34 = t31 * t33;
		t37 = t27 * n;
		t38 = t37 * M_PI;
		t39 = t32 * t19;
		t40 = t5 * t39;
		t41 = t40 * t10;
		t44 = t20 * t10;
		t47 = t21 * t21;
		t48 = t27 * t47;
		t49 = t48 * t6;
		t50 = t6 * t19;
		t52 = t15 * t6;
		t53 = t52 * t19;
		t56 = t30 * startBlockY;
		t57 = t56 * n;
		t58 = M_PI * t19;
		t59 = t21 * t6;
		t60 = t58 * t59;
		t64 = t19 * t10;
		t65 = t64 * t21;
		t68 = t27 * t18;
		t71 = t68 * t22;
		t74 = -t7 * t11 - 0.4e1 * t17 * t23 - t29 + 0.6e1 * t14 * t34 - 0.2e1 * t38 * t41 - 0.2e1 * t1 * t44 - t49 + 0.2e1 * t50 - 0.4e1 * t14 * t53 - 0.4e1 * t57 * t60 - 0.2e1 * t1 * t18 * t65 + 0.2e1 * t68 * t65 + 0.2e1 * t14 * t71;
		t75 = t30 * t18;
		t76 = t75 * t22;
		t78 = startBlockY * t6;
		t79 = t78 * t19;
		t84 = t15 * t18;
		t85 = t84 * t22;
		t87 = t27 * startBlockY;
		t91 = M_PI * t32;
		t92 = t91 * t59;
		t95 = t15 * t5;
		t96 = t39 * t10;
		t97 = t95 * t96;
		t99 = t95 * t64;
		t101 = M_PI * t6;
		t104 = t75 * t64;
		t106 = t30 * t11;
		t107 = t106 * t50;
		t109 = t15 * startBlockY;
		t110 = t109 * n;
		t111 = M_PI * t11;
		t112 = t6 * t32;
		t113 = t111 * t112;
		t116 = M_PI * t27;
		t117 = t13 * t116;
		t118 = t87 * n;
		t121 = 0.2e1 * t76 - 0.4e1 * t17 * t79 - 0.2e1 * t7 * t21 + 0.4e1 * t85 - 0.6e1 * t87 * t1 * t32 - 0.2e1 * t57 * t92 - 0.4e1 * t97 - 0.12e2 * t99 + 0.4e1 * t16 * t101 + 0.2e1 * t104 + 0.2e1 * t107 + 0.2e1 * t110 * t113 + t117 + 0.4e1 * t118 * t60;
		t123 = t11 * t6;
		t126 = t5 * t19;
		t127 = t126 * t22;
		t130 = M_PI * t7;
		t132 = t18 * t10;
		t135 = t27 * t5;
		t136 = t135 * t10;
		t138 = t68 * t64;
		t143 = t130 * t47;
		t145 = t30 * t7;
		t146 = t145 * t11;
		t150 = t5 * t32;
		t151 = t150 * t22;
		t155 = 0.2e1 * t1 * t123 - 0.6e1 * t127 - t13 * t101 + t8 * t130 - 0.2e1 * t1 * t132 - 0.2e1 * t136 + 0.2e1 * t138 + 0.6e1 * t118 * t91 * t11 + t57 * t143 + t14 * t146 - 0.4e1 * t17 * t41 - 0.6e1 * t151 + 0.4e1 * t1 * t59;
		t156 = t47 * t6;
		t159 = t95 * t10;
		t161 = M_PI * t47;
		t162 = t161 * t112;
		t165 = t40 * t22;
		t175 = t130 * t11;
		t177 = t91 * t6;
		t183 = 0.2e1 * t1 * t156 - 0.4e1 * t159 - 0.2e1 * t110 * t162 - 0.2e1 * t38 * t165 - 0.2e1 * t165 - t7 + 0.2e1 * t110 * t143 + 0.2e1 * t118 * t92 + 0.2e1 * t38 * t79 + t13 * t175 - t57 * t177 + t118 * t177 - t14 * t49 - 0.12e2 * t95 * t65;
		t186 = t59 * t19;
		t189 = t32 * t32;
		t190 = M_PI * t189;
		t193 = t95 * t22;
		t199 = t5 * t10;
		t202 = t30 * t21;
		t203 = t202 * t6;
		t206 = t135 * t96;
		t209 = t135 * t22;
		t212 = t95 * t33;
		t214 = t106 * t112;
		t218 = t31 * t64;
		t220 = 0.16e2 * t17 * t186 + t118 * t190 * t11 + 0.4e1 * t14 * t193 - 0.6e1 * t31 * t65 + 0.2e1 * t23 - 0.2e1 * t1 * t199 - 0.2e1 * t14 * t203 - 0.2e1 * t14 * t206 - 0.2e1 * t14 * t209 - 0.12e2 * t212 + t214 + 0.2e1 * t13 * t116 * t21 - 0.6e1 * t218;
		t226 = t202 * t50;
		t229 = t30 * t47;
		t230 = t229 * t50;
		t233 = t68 * t10;
		t236 = t31 * t96;
		t241 = t111 * t50;
		t244 = t101 * t11;
		t247 = t123 * t32;
		t250 = t156 * t19;
		t253 = t57 * t113 + t28 + 0.12e2 * t14 * t99 + 0.2e1 * t37 * t101 - 0.4e1 * t14 * t226 + 0.2e1 * t230 + 0.4e1 * t226 + 0.2e1 * t14 * t233 + 0.2e1 * t14 * t236 + 0.4e1 * t1 * t50 + 0.4e1 * t110 * t241 + 0.4e1 * t16 * t244 + 0.4e1 * t17 * t247 - 0.2e1 * t14 * t250;
		t256 = t161 * t50;
		t260 = t101 * t21;
		t263 = t123 * t19;
		t268 = t31 * t22;
		t272 = t229 * t112;
		t273 = t84 * t64;
		t275 = M_PI * t30;
		t276 = t13 * t275;
		t280 = t15 * t21;
		t281 = t280 * t50;
		t283 = t118 * t162 - 0.4e1 * t110 * t256 - t118 * t113 + 0.4e1 * t37 * t260 - 0.2e1 * t14 * t263 - 0.4e1 * t14 * t85 - 0.2e1 * t268 - 0.2e1 * t14 * t136 + t272 + 0.4e1 * t273 + 0.2e1 * t276 * t165 - 0.4e1 * t193 + 0.8e1 * t281;
		t285 = t30 * t6;
		t286 = t285 * t32;
		t288 = t156 * t32;
		t291 = t101 * t19;
		t294 = t48 * t50;
		t296 = t30 * n;
		t297 = t296 * M_PI;
		t298 = t132 * t21;
		t301 = t28 * t50;
		t303 = t48 * t112;
		t305 = t28 * t112;
		t310 = -t8 * t175 - t14 * t286 + 0.2e1 * t1 * t288 + 0.8e1 * t16 * t291 - 0.2e1 * t294 + t156 - 0.2e1 * t297 * t298 - 0.2e1 * t301 - t303 - 0.2e1 * t199 - t305 + 0.2e1 * t71 - 0.2e1 * t276 * t23 - t9 * t288;
		t314 = t59 * t32;
		t321 = t135 * t33;
		t326 = t33 * t21;
		t333 = t27 * t21;
		t334 = t333 * t50;
		t338 = t15 * t11;
		t339 = t338 * t50;
		t341 = t27 * t189;
		t342 = 0.4e1 * t1 * t314 - 0.2e1 * t87 * t1 * t21 - t7 * t47 - 0.6e1 * t321 - 0.4e1 * t87 * t1 * t39 - 0.6e1 * t135 * t326 - t14 * t247 + 0.2e1 * t14 * t165 - 0.2e1 * t236 - 0.4e1 * t334 + 0.2e1 * t57 * t241 + 0.4e1 * t339 + t341;
		t343 = t15 * t47;
		t344 = t343 * t112;
		t346 = t343 * t50;
		t348 = M_PI * t15;
		t349 = t13 * t348;
		t365 = t161 * t6;
		t372 = M_PI * startBlockY * t6;
		t374 = 0.2e1 * t344 + 0.4e1 * t346 + 0.12e2 * t349 * t151 - 0.2e1 * t14 * t44 - 0.2e1 * t57 * t256 + 0.4e1 * t1 * t250 + 0.8e1 * t1 * t186 + 0.8e1 * t17 * t263 - 0.2e1 * t57 * t260 - 0.6e1 * t34 - t13 * t365 - 0.2e1 * t1 * t41 - 0.2e1 * t14 * t107 - t296 * t372;
		t379 = t150 * t10;
		t388 = t341 * t47;
		t390 = t96 * t21;
		t394 = M_PI * t18 * t10;
		t402 = -0.2e1 * t14 * t23 + t57 * t244 - 0.6e1 * t297 * t379 + 0.2e1 * t296 * t244 - 0.2e1 * t297 * t23 - t1 * t78 + 0.2e1 * t59 + t14 * t388 - 0.2e1 * t135 * t390 - 0.2e1 * t296 * t394 - 0.2e1 * t110 * t175 - t87 * t1 - 0.2e1 * t297 * t44;
		t403 = t27 * t39;
		t404 = t403 * t47;
		t409 = t15 * t7;
		t410 = t409 * t47;
		t414 = t333 * t112;
		t425 = t130 * t21;
		t429 = t341 * t11;
		t431 = 0.4e1 * t14 * t404 + 0.2e1 * t296 * t365 - 0.2e1 * t410 - 0.2e1 * t16 * t372 - 0.2e1 * t414 - 0.4e1 * t17 * t165 + t112 + t8 * t143 - 0.6e1 * t38 * t379 - t14 * t214 + 0.4e1 * t13 * t116 * t19 + 0.2e1 * t8 * t425 + 0.2e1 * t132 + t14 * t429;
		t447 = t126 * t10;
		t450 = t199 * t21;
		t455 = M_PI * t5 * t10;
		t460 = t1 * t5;
		t463 = -0.6e1 * t118 * t91 * t47 - 0.6e1 * t38 * t151 - 0.2e1 * t297 * t41 - 0.4e1 * t17 * t298 - 0.6e1 * t297 * t151 + t48 - 0.6e1 * t135 * t65 - 0.6e1 * t38 * t447 - 0.2e1 * t1 * t450 + t9 * t247 - 0.2e1 * t37 * t455 - 0.6e1 * t1 * t379 - 0.6e1 * t460 * t326;
		t464 = t135 * t64;
		t467 = t403 * t21;
		t491 = -0.6e1 * t464 + t37 * t372 + 0.8e1 * t467 - 0.6e1 * t1 * t447 - 0.4e1 * t14 * t273 + 0.2e1 * t296 * t177 + 0.2e1 * t37 * t244 - 0.6e1 * t460 * t65 + 0.6e1 * t14 * t151 + 0.8e1 * t16 * t260 - 0.2e1 * t460 * t390 + 0.4e1 * t296 * t291 - 0.12e2 * t95 * t326 - 0.2e1 * t37 * t394;
		t501 = t202 * t112;
		t512 = t1 * t7;
		t518 = t27 * t19;
		t519 = t518 * t11;
		t522 = -0.4e1 * t16 * t394 - 0.4e1 * t110 * t260 - 0.4e1 * t95 * t390 - 0.2e1 * t1 * t79 - 0.2e1 * t14 * t501 + 0.2e1 * t297 * t288 - 0.2e1 * t14 * t314 + 0.4e1 * t84 * t65 + 0.6e1 * t276 * t127 + t56 * t512 - 0.2e1 * t38 * t298 + 0.2e1 * t37 * t365 + 0.4e1 * t14 * t519;
		t540 = t84 * t10;
		t550 = -0.6e1 * t14 * t464 - t8 * t177 + 0.2e1 * t110 * t244 - 0.4e1 * t14 * t339 - 0.2e1 * t1 * t298 + 0.12e2 * t14 * t212 - 0.2e1 * t38 * t44 - t13 * t177 - 0.4e1 * t87 * t1 * t19 + 0.4e1 * t540 - 0.2e1 * t297 * t165 + 0.2e1 * t296 * t101 - 0.2e1 * t13 * t394 - 0.2e1 * t9 * t314;
		t557 = t338 * t112;
		t567 = t518 * t21;
		t582 = -0.4e1 * t17 * t450 - 0.2e1 * t14 * t557 + 0.4e1 * t110 * t425 + 0.2e1 * t14 * t268 + 0.2e1 * t38 * t247 + t13 * t143 + 0.8e1 * t14 * t567 - 0.4e1 * t9 * t186 + 0.2e1 * t13 * t348 * t7 - 0.2e1 * t209 + 0.4e1 * t14 * t97 + 0.2e1 * t57 * t425 - 0.12e2 * t17 * t379;
		t587 = t145 * t47;
		t598 = t145 * t21;
		t609 = -t57 * t162 - 0.4e1 * t14 * t334 - t57 * t175 + t14 * t587 - 0.4e1 * t14 * t540 - 0.12e2 * t17 * t151 - t146 + 0.2e1 * t14 * t410 - 0.12e2 * t118 * t91 * t21 + 0.2e1 * t14 * t598 + 0.4e1 * t38 * t263 - 0.2e1 * t297 * t450 + 0.4e1 * t14 * t159 + 0.2e1 * t38 * t288;
		t611 = t52 * t32;
		t614 = t27 * t32;
		t615 = t614 * t21;
		t618 = M_PI * t39;
		t634 = t285 * t19;
		t637 = t280 * t112;
		t639 = t27 * t6;
		t640 = t639 * t32;
		t642 = t280 * t6;
		t645 = -0.2e1 * t14 * t611 + 0.12e2 * t14 * t615 + 0.4e1 * t118 * t618 * t11 - 0.2e1 * t118 * t241 - t145 + 0.4e1 * t118 * t58 * t11 + 0.2e1 * t118 * t256 - 0.2e1 * t14 * t414 + 0.4e1 * t13 * t116 * t39 - 0.2e1 * t14 * t634 + 0.4e1 * t637 - t14 * t640 - 0.4e1 * t14 * t642;
		t654 = t333 * t6;
		t673 = -0.2e1 * t13 * t260 - t13 * t275 * t6 + 0.4e1 * t296 * t260 - 0.6e1 * t117 * t151 - 0.2e1 * t14 * t654 + 0.6e1 * t14 * t218 + 0.4e1 * t349 * t165 + 0.4e1 * t38 * t314 - 0.12e2 * t17 * t447 - 0.12e2 * t17 * t127 + 0.2e1 * t13 * t455 + t27 - 0.4e1 * t349 * t23 - 0.6e1 * t31 * t326;
		t682 = t409 * t11;
		t702 = t13 * t275 * t7 - 0.2e1 * t117 * t165 + 0.6e1 * t276 * t151 - 0.2e1 * t682 - 0.8e1 * t14 * t281 + 0.2e1 * t297 * t247 - 0.2e1 * t110 * t365 - 0.2e1 * t14 * t230 - 0.4e1 * t14 * t637 + 0.4e1 * t38 * t250 - t87 * t1 * t47 + 0.2e1 * t117 * t23 - 0.2e1 * t8 * t260;
		t705 = t518 * t47;
		t716 = t343 * t6;
		t719 = t614 * t11;
		t721 = t403 * t11;
		t726 = t409 * t21;
		t728 = t338 * t6;
		t731 = 0.2e1 * t13 * t425 + 0.4e1 * t14 * t705 - t13 * t116 * t6 - 0.8e1 * t110 * t60 - 0.4e1 * t16 * t455 - 0.2e1 * t14 * t76 - 0.2e1 * t14 * t716 + 0.6e1 * t719 + 0.4e1 * t721 + 0.4e1 * t297 * t314 + 0.4e1 * t642 - 0.4e1 * t726 + 0.2e1 * t728 + 0.2e1 * t716;
		t734 = t106 * t6;
		t735 = t229 * t6;
		t740 = t614 * t47;
		t742 = t31 * t10;
		t744 = 0.2e1 * t203 + t734 + t735 + t247 + 0.2e1 * t263 - 0.2e1 * t598 - 0.2e1 * t450 - t587 - t14 * t272 + 0.6e1 * t740 + t429 - 0.2e1 * t742 + t286;
		t747 = t75 * t10;
		t753 = t341 * t21;
		t761 = -0.2e1 * t41 - 0.6e1 * t379 + 0.2e1 * t747 + 0.4e1 * t186 + 0.2e1 * t314 + 0.2e1 * t611 - 0.2e1 * t654 + 0.2e1 * t753 - t14 * t735 + 0.2e1 * t250 + t288 + 0.2e1 * t1 * t6 + 0.4e1 * t705 + 0.4e1 * t519;
		t771 = t639 * t19;
		t779 = 0.2e1 * t233 + 0.12e2 * t615 + 0.8e1 * t567 + 0.6e1 * t14 * t127 + 0.2e1 * t634 - 0.2e1 * t771 + 0.4e1 * t53 + 0.4e1 * t404 + t388 + 0.2e1 * t44 + 0.8e1 * t38 * t186 + 0.2e1 * t298 + t123;
		t801 = -0.4e1 * t14 * t346 - 0.6e1 * t447 - 0.6e1 * t38 * t127 + 0.6e1 * t14 * t740 - t14 * t29 + 0.2e1 * t1 * t247 - t14 * t734 - 0.4e1 * t17 * t44 - t640 + t8 * t244 - 0.2e1 * t14 * t344 - 0.4e1 * t110 * t92 + 0.2e1 * t557 - 0.2e1 * t31 * t390;
		t830 = -0.4e1 * t118 * t58 * t47 - 0.4e1 * t14 * t186 + 0.2e1 * t9 * t263 - 0.2e1 * t9 * t250 + 0.2e1 * t14 * t138 - 0.2e1 * t14 * t104 + 0.4e1 * t17 * t288 + 0.2e1 * t501 + 0.6e1 * t13 * t116 * t32 + 0.2e1 * t37 * t177 + 0.6e1 * t14 * t379 + 0.2e1 * t14 * t41 - t118 * t190 * t47;
		t854 = -t87 * t1 * t189 - 0.6e1 * t14 * t321 + 0.2e1 * t333 + t13 * t130 + 0.2e1 * t75 * t65 + 0.8e1 * t17 * t314 + 0.2e1 * t109 * t512 + 0.4e1 * t297 * t250 - t8 * t365 + 0.4e1 * t14 * t721 + t6 + 0.4e1 * t16 * t365 - 0.6e1 * t297 * t447 - 0.2e1 * t14 * t301;
		t883 = 0.4e1 * t37 * t291 - 0.2e1 * t296 * t455 + 0.8e1 * t17 * t250 - 0.2e1 * t118 * t190 * t21 + t87 * t1 * t11 + 0.12e2 * t349 * t127 + 0.6e1 * t14 * t719 + 0.2e1 * t52 + 0.2e1 * t14 * t450 - 0.2e1 * t38 * t23 + 0.4e1 * t297 * t263 + 0.2e1 * t1 * t112 - 0.2e1 * t297 * t79;
		t910 = -0.2e1 * t13 * t291 - 0.6e1 * t117 * t127 + t285 - 0.2e1 * t13 * t348 * t6 + 0.8e1 * t297 * t186 - t13 * t244 - t57 * t365 - 0.4e1 * t118 * t618 * t47 + 0.2e1 * t14 * t753 + t13 * t116 * t11 + 0.2e1 * t118 * t260 + 0.6e1 * t14 * t447 - 0.2e1 * t206 - 0.8e1 * t118 * t618 * t21;
		t936 = -0.2e1 * t14 * t771 + 0.2e1 * t14 * t742 + t13 * t116 * t47 - 0.6e1 * t297 * t127 - 0.2e1 * t14 * t728 - t14 * t288 - 0.2e1 * t38 * t450 + 0.6e1 * t614 - 0.2e1 * t14 * t294 + 0.4e1 * t518 - 0.2e1 * t110 * t177 - 0.2e1 * t14 * t747 - 0.8e1 * t118 * t58 * t21;
		t957 = 0.2e1 * t14 * t682 + 0.4e1 * t14 * t726 + 0.4e1 * t16 * t177 + t13 * t116 * t189 + 0.4e1 * t403 - 0.2e1 * t14 * t298 - t14 * t303 - t118 * t244 - t639 + 0.4e1 * t1 * t263 - 0.2e1 * t409 + t118 * t365 - t14 * t305 + 0.8e1 * t14 * t467;
		t969 = pow(t5 + 0.1e1 + t19, 0.2e1);
		t974 = pow(t5 - 0.1e1 - t19, 0.2e1);
		t978 = n * n;
		t982 = M_PI * M_PI;

		u2b = deltaRho * t3 * (t910 + t74 + t582 + t310 + t830 + t491 + t883 + t645 + t342 + t957 + t283 + t673 + t220 + t121 + t183 + t609 + t550 + t731 + t374 + t702 + t402 + t431 + t854 + t522 + t761 + t744 + t779 + t463 + t155 + t253 + t936 + t801) / t10 / (0.1e1 + t21) / t969 / t974 / t26 / t978 / n / t982 / M_PI / viscosity / (0.1e1 + t15) / 0.2e1;

		t1 = n * M_PI;
		t2 = 0.1e1 / cosh(t1);
		t3 = startBlockY * n;
		t4 = t3 * M_PI;
		t5 = tanh(t4);
		t6 = t2 * t5;
		t7 = 0.1e1 / cosh(t4);
		t8 = tanh(t1);
		t9 = t8 * t7;
		t13 = sin(t1 * dx);
		t15 = y * n;
		t16 = t15 * M_PI;
		t17 = tanh(t16);
		t18 = t17 * t17;
		t19 = t18 * t2;
		t20 = t19 * startBlockY;
		t21 = t8 * t8;
		t22 = t21 * t5;
		t23 = t1 * t22;
		t25 = t2 * t2;
		t26 = t25 * t2;
		t27 = t18 * t26;
		t29 = t1 * t5;
		t31 = 0.1e1 / cosh(t16);
		t32 = t31 * t31;
		t33 = t32 * n;
		t34 = M_PI * t2;
		t35 = t34 * t21;
		t38 = M_PI * t18;
		t41 = t17 * t26;
		t45 = t17 * startBlockY;
		t46 = t45 * n;
		t47 = M_PI * t21;
		t48 = t47 * t7;
		t51 = t19 * t8;
		t53 = t19 * t21;
		t57 = M_PI * t17;
		t62 = t18 * n;
		t65 = -t20 * t23 + t27 * startBlockY * t29 + 0.2e1 * t33 * t35 + t15 * t38 * t26 + 0.2e1 * t41 * startBlockY * t29 - 0.6e1 * t46 * t48 + 0.4e1 * t51 + 0.2e1 * t53 - t15 * t38 * t2 - 0.2e1 * t15 * t57 * t2 - 0.2e1 * t27 + 0.2e1 * t62 * t35;
		t66 = t62 * M_PI;
		t67 = t2 * t21;
		t68 = t67 * t5;
		t71 = t32 * t2;
		t72 = t71 * startBlockY;
		t73 = t8 * t5;
		t74 = t1 * t73;
		t77 = t17 * n;
		t78 = t77 * M_PI;
		t82 = t17 * t21 * t7;
		t87 = t18 * startBlockY;
		t88 = t87 * n;
		t89 = M_PI * t8;
		t90 = t89 * t7;
		t93 = t21 * t8;
		t97 = t2 * t8;
		t98 = t97 * t5;
		t102 = t17 * t2;
		t103 = t102 * startBlockY;
		t110 = 0.2e1 * t66 * t68 - 0.2e1 * t72 * t74 + 0.4e1 * t78 * t68 + 0.6e1 * t16 * t82 + 0.2e1 * t1 * t68 - 0.3e1 * t88 * t90 - 0.2e1 * t93 * t7 + 0.2e1 * t2 + 0.8e1 * t78 * t98 - 0.6e1 * t9 - 0.2e1 * t103 * t29 - 0.4e1 * t103 * t74 - 0.2e1 * t103 * t23;
		t113 = t25 * t8 * t7;
		t116 = t102 * t73;
		t123 = M_PI * t26;
		t127 = M_PI * t93 * t7;
		t132 = M_PI * t25 * t7;
		t134 = t33 * M_PI;
		t140 = -t16 * t113 - t16 * t68 - 0.4e1 * t16 * t116 - 0.4e1 * t17 * t7 - 0.2e1 * t16 * t98 + t15 * t123 * t5 + t15 * t127 + 0.3e1 * t15 * t48 - t15 * t132 + 0.2e1 * t134 * t68 + 0.4e1 * t134 * t98 + 0.4e1 * t102;
		t141 = t34 * t5;
		t145 = t18 * t25;
		t146 = t145 * t7;
		t148 = t32 * startBlockY;
		t149 = t148 * n;
		t154 = t25 * t7;
		t156 = t102 * t22;
		t159 = t18 * t93 * t7;
		t166 = t102 * t21;
		t169 = t102 * t8;
		t172 = 0.2e1 * t62 * t141 - t72 * t23 - t16 * t146 - 0.3e1 * t149 * t48 - 0.3e1 * t88 * t48 + 0.2e1 * t154 + 0.4e1 * t156 + t16 * t159 + t15 * t38 * t7 + 0.4e1 * t77 * t34 - t15 * t34 - 0.2e1 * t16 * t166 - 0.4e1 * t16 * t169;
		t177 = t17 * t8 * t7;
		t180 = M_PI * t32;
		t183 = startBlockY * t25;
		t184 = t183 * t7;
		t186 = t1 * t8;
		t192 = t32 * t26;
		t193 = t192 * t5;
		t196 = t32 * t21 * t7;
		t201 = 0.2e1 * t6 + 0.6e1 * t16 * t177 - t15 * t180 * t26 + t66 * t184 - 0.2e1 * t20 * t186 + 0.4e1 * t98 - 0.12e2 * t177 + 0.8e1 * t169 - 0.2e1 * t193 - 0.6e1 * t196 + 0.2e1 * t1 * t2 + 0.2e1 * t67;
		t202 = t71 * t73;
		t205 = M_PI * t7;
		t218 = t145 * t9;
		t226 = 0.2e1 * t16 * t202 + t15 * t205 - t15 * t180 * t7 + t15 * t180 * t2 + 0.4e1 * t202 + t15 * t123 - 0.3e1 * t149 * t90 - 0.2e1 * t33 * t132 + t134 * t184 - t16 * t218 - 0.2e1 * t72 * t186 - 0.2e1 * t1 * t154 - 0.2e1 * t134 * t113;
		t228 = t1 * t7;
		t235 = t2 * startBlockY;
		t236 = t235 * n;
		t244 = t19 * t22;
		t248 = t26 * startBlockY;
		t252 = -t148 * t228 + 0.2e1 * t33 * t141 - t20 * t29 + 0.2e1 * t1 * t6 - t236 * t47 * t5 - t235 * t29 - 0.2e1 * t236 * t89 * t5 - t3 * t205 - t16 * t244 - 0.2e1 * t20 * t74 + t248 * t1 + 0.4e1 * t66 * t98;
		t257 = t34 * t8;
		t265 = t17 * t25;
		t266 = t265 * t9;
		t269 = t1 * t21;
		t277 = t102 * t5;
		t280 = -t235 * t1 - t149 * t127 + 0.2e1 * t1 * t67 + 0.4e1 * t62 * t257 - t72 * t29 - 0.2e1 * t16 * t51 + 0.4e1 * t1 * t97 - 0.2e1 * t16 * t266 - 0.2e1 * t103 * t269 - 0.2e1 * t45 * t228 + 0.4e1 * t77 * t141 - t72 * t269 - 0.2e1 * t16 * t277;
		t286 = t265 * t7;
		t289 = t71 * t21;
		t291 = t41 * t5;
		t296 = t32 * t25;
		t297 = t296 * t7;
		t307 = -0.2e1 * t1 * t113 - 0.2e1 * t16 * t286 + t16 * t289 + 0.2e1 * t16 * t291 + 0.2e1 * t71 - t16 * t193 + t16 * t297 - t3 * t127 - 0.2e1 * t16 * t156 - 0.2e1 * t235 * t186 - 0.3e1 * t3 * t90 - t19 * t4;
		t310 = t18 * t8 * t7;
		t331 = t27 * t4 + 0.3e1 * t16 * t310 + 0.2e1 * t15 * t57 * t26 - 0.2e1 * t192 + 0.8e1 * t77 * t257 + 0.2e1 * t78 * t184 - t16 * t53 + 0.4e1 * t33 * t257 + 0.3e1 * t15 * t90 - t15 * t141 - 0.4e1 * t78 * t113 + 0.2e1 * t244 - t20 * t269;
		t334 = t19 * t5;
		t339 = t18 * t21 * t7;
		t341 = t27 * t5;
		t347 = t71 * t8;
		t350 = t32 * t8 * t7;
		t352 = -0.6e1 * t310 + 0.2e1 * t334 + 0.2e1 * t146 - 0.2e1 * t159 - 0.6e1 * t339 - 0.2e1 * t341 + 0.2e1 * t113 + 0.2e1 * t68 + 0.2e1 * t289 + 0.4e1 * t166 + 0.4e1 * t347 - 0.6e1 * t350;
		t353 = t71 * t5;
		t356 = t32 * t93 * t7;
		t362 = t17 * t93 * t7;
		t366 = t19 * t73;
		t369 = t183 * t9;
		t376 = 0.2e1 * t353 - 0.2e1 * t356 + 0.2e1 * t297 + 0.4e1 * t277 + 0.4e1 * t286 - 0.4e1 * t362 - 0.12e2 * t82 - 0.4e1 * t291 + 0.4e1 * t366 + 0.2e1 * t218 + 0.2e1 * t78 * t369 + 0.2e1 * t62 * t34 - 0.6e1 * t21 * t7;
		t399 = -0.2e1 * t62 * t132 - 0.2e1 * t66 * t113 + 0.2e1 * t15 * t57 * t7 - 0.3e1 * t16 * t350 + t16 * t353 - 0.2e1 * t32 * t7 - 0.4e1 * t41 + 0.4e1 * t97 + 0.8e1 * t116 - 0.4e1 * t103 * t186 - 0.3e1 * t3 * t48 - t87 * t228;
		t405 = t71 * t22;
		t409 = t296 * t9;
		t421 = -0.2e1 * t26 * t5 + t192 * startBlockY * t29 + t134 * t369 + 0.2e1 * t405 + 0.4e1 * t266 + 0.2e1 * t19 + 0.2e1 * t409 - 0.2e1 * t18 * t7 - t16 * t334 + 0.2e1 * t33 * t34 + 0.4e1 * t77 * t35 + 0.4e1 * t1 * t98 + t66 * t369;
		t442 = 0.2e1 * t16 * t362 - t15 * t35 - 0.2e1 * t15 * t257 - 0.2e1 * t46 * t127 - 0.6e1 * t46 * t90 - t88 * t127 - 0.3e1 * t16 * t196 + t16 * t409 + 0.3e1 * t16 * t339 - 0.2e1 * t7 - 0.2e1 * t102 * t4 - t235 * t269;
		t460 = -0.2e1 * t16 * t366 + t16 * t341 - 0.4e1 * t77 * t132 + t1 * t184 + 0.2e1 * t16 * t347 + t4 * t113 + t248 * t29 - t71 * t4 - 0.2e1 * t26 + 0.2e1 * t41 * t4 - t16 * t356 + t16 * t405 + t192 * t4;
		t471 = n * n;
		t474 = M_PI * M_PI;
		t479 = pow(t2 + 0.1e1 + t8, 0.2e1);
		t483 = pow(t2 - 0.1e1 - t8, 0.2e1);

		u3a = -(t2 + t6 - t7 - t9) * deltaRho * t13 * (t65 + t110 + t140 + t172 + t201 + t226 + t252 + t280 + t307 + t331 + t352 + t376 + t399 + t421 + t442 + t460) / t7 / (0.1e1 + t5) / t471 / n / t474 / M_PI / t479 / t483 / t31 / (0.1e1 + t17) / 0.2e1;

		t1 = n * M_PI;
		t3 = sin(t1 * dx);
		t5 = y * n;
		t6 = t5 * M_PI;
		t7 = tanh(t6);
		t8 = t7 * t7;
		t9 = 0.1e1 / cosh(t1);
		t10 = t8 * t9;
		t11 = tanh(t1);
		t12 = t11 * t11;
		t13 = startBlockY * n;
		t14 = t13 * M_PI;
		t15 = 0.1e1 / cosh(t14);
		t16 = t12 * t15;
		t17 = t10 * t16;
		t20 = t9 * t9;
		t21 = t20 * t20;
		t22 = t8 * t21;
		t23 = t15 * t15;
		t24 = t22 * t23;
		t26 = tanh(t14);
		t27 = t22 * t26;
		t30 = t10 * t15;
		t33 = 0.1e1 / cosh(t6);
		t34 = t33 * t33;
		t35 = t34 * t11;
		t36 = t35 * t26;
		t39 = t34 * n;
		t40 = t39 * M_PI;
		t41 = t26 * t20;
		t42 = t41 * t11;
		t45 = t8 * n;
		t46 = t45 * M_PI;
		t47 = t26 * t26;
		t48 = t47 * t20;
		t49 = t48 * t11;
		t52 = t23 * t20;
		t54 = t34 * t47;
		t55 = t54 * t20;
		t57 = t7 * t9;
		t58 = t15 * t26;
		t59 = t57 * t58;
		t64 = M_PI * t7;
		t65 = t5 * t64;
		t66 = t9 * t11;
		t67 = t66 * t58;
		t70 = t8 * t47;
		t71 = t20 * t12;
		t72 = t70 * t71;
		t74 = 0.6e1 * t6 * t17 + t6 * t24 + 0.2e1 * t6 * t27 + 0.2e1 * t6 * t30 + 0.8e1 * t6 * t36 + 0.8e1 * t40 * t42 + 0.4e1 * t46 * t49 + 0.2e1 * t52 - t6 * t55 + 0.4e1 * t6 * t59 - 0.2e1 * t6 * t49 + 0.12e2 * t65 * t67 - t6 * t72;
		t75 = t34 * startBlockY;
		t76 = t75 * n;
		t77 = t12 * t11;
		t78 = M_PI * t77;
		t82 = t9 * t15;
		t83 = t82 * t26;
		t86 = t48 * t12;
		t88 = M_PI * t12;
		t93 = t7 * startBlockY;
		t94 = t93 * n;
		t95 = M_PI * t11;
		t96 = t95 * t41;
		t99 = t20 * t9;
		t100 = t8 * t99;
		t101 = t100 * t58;
		t104 = t34 * t26;
		t105 = t104 * t71;
		t108 = t7 * t20;
		t109 = t108 * t11;
		t112 = t7 * n;
		t113 = t112 * M_PI;
		t114 = startBlockY * t20;
		t115 = t114 * t11;
		t118 = t1 * t9;
		t119 = t11 * t15;
		t120 = t119 * t26;
		t123 = t99 * t11;
		t124 = t123 * t58;
		t127 = t77 * t15;
		t128 = t10 * t127;
		t130 = t52 * t12;
		t133 = -0.8e1 * t76 * t78 * t26 - 0.2e1 * t40 * t83 - t6 * t86 - 0.12e2 * t76 * t88 * t26 + 0.2e1 * t54 - 0.8e1 * t94 * t96 - 0.2e1 * t6 * t101 - 0.2e1 * t6 * t105 - 0.4e1 * t6 * t109 - 0.4e1 * t113 * t115 - 0.6e1 * t118 * t120 - 0.2e1 * t6 * t124 - 0.4e1 * t128 + 0.2e1 * t40 * t130;
		t138 = t8 * startBlockY;
		t139 = t138 * n;
		t140 = M_PI * t21;
		t141 = t140 * t47;
		t146 = M_PI * t20;
		t151 = t57 * t127;
		t153 = t57 * t119;
		t157 = t100 * t119;
		t159 = t34 * t9;
		t160 = t159 * t119;
		t164 = 0.6e1 * t76 * t88 * t23 + t139 * t141 + 0.4e1 * t101 - t1 * t114 - t75 * t1 + 0.2e1 * t45 * t146 + 0.2e1 * t1 * t71 - 0.8e1 * t151 - 0.24e2 * t153 + 0.4e1 * t112 * t146 + 0.4e1 * t157 - 0.12e2 * t160 + 0.6e1 * t6 * t67;
		t165 = t159 * t58;
		t167 = t159 * t127;
		t169 = t127 * t26;
		t173 = t34 * t99;
		t174 = t173 * t119;
		t178 = t7 * t99;
		t179 = t178 * t58;
		t186 = t173 * t58;
		t188 = t9 * t77;
		t189 = t188 * t58;
		t192 = t99 * t15;
		t193 = t192 * t26;
		t196 = t20 * t11;
		t197 = t104 * t196;
		t199 = -0.4e1 * t165 - 0.4e1 * t167 - 0.4e1 * t159 * t169 - 0.4e1 * t105 + 0.2e1 * t6 * t174 + 0.2e1 * t20 + 0.8e1 * t179 + 0.4e1 * t174 + 0.2e1 * t6 * t128 - 0.2e1 * t6 * t157 + 0.4e1 * t186 - 0.4e1 * t113 * t189 - 0.4e1 * t113 * t193 - 0.8e1 * t197;
		t205 = t10 * t119;
		t208 = t35 * t47;
		t211 = M_PI * t23;
		t212 = t211 * t71;
		t216 = t7 * t23;
		t217 = t216 * t71;
		t219 = t8 * t26;
		t220 = t219 * t71;
		t222 = t57 * t16;
		t225 = t10 * t58;
		t230 = t7 * t26;
		t231 = t230 * t71;
		t233 = 0.4e1 * t173 * t120 + t5 * t140 + 0.6e1 * t6 * t205 + 0.4e1 * t6 * t208 + t139 * t212 + 0.2e1 * t94 * t212 + 0.4e1 * t217 + 0.4e1 * t220 - 0.24e2 * t222 + 0.2e1 * t72 - 0.4e1 * t225 - 0.6e1 * t75 * t1 * t12 + 0.8e1 * t231;
		t234 = t9 * t12;
		t235 = t234 * t15;
		t238 = t16 * t26;
		t241 = t66 * t15;
		t244 = t52 * t11;
		t250 = M_PI * t26 * t20;
		t253 = t211 * t20;
		t255 = t88 * t41;
		t258 = M_PI * t34;
		t259 = t5 * t258;
		t262 = t88 * t20;
		t266 = t12 * t12;
		t267 = t34 * t266;
		t268 = t267 * t23;
		t272 = -0.6e1 * t1 * t235 - 0.6e1 * t118 * t238 - 0.6e1 * t1 * t241 + 0.4e1 * t40 * t244 + 0.4e1 * t76 * t96 - 0.2e1 * t139 * t250 + t139 * t253 + 0.2e1 * t76 * t255 + t259 + 0.16e2 * t113 * t42 + t76 * t262 - 0.12e2 * t10 * t238 + t6 * t268 + 0.12e2 * t6 * t153;
		t276 = t8 * t23;
		t277 = t276 * t196;
		t279 = M_PI * t47;
		t280 = t279 * t20;
		t294 = t219 * t20;
		t299 = 0.4e1 * t112 * t253 + 0.4e1 * t277 - t5 * t280 - 0.6e1 * t46 * t235 + 0.2e1 * t1 * t52 - 0.12e2 * t67 - t5 * t146 - t75 * t1 * t47 - 0.2e1 * t1 * t115 + 0.12e2 * t6 * t222 - 0.2e1 * t6 * t294 - 0.4e1 * t82 - t13 * t280;
		t300 = t234 * t58;
		t308 = t140 * t26;
		t313 = M_PI * startBlockY * t20;
		t321 = t279 * t71;
		t324 = M_PI * t99 * t15;
		t329 = 0.6e1 * t6 * t300 - 0.2e1 * t40 * t124 - 0.2e1 * t1 * t192 + t13 * t141 + 0.2e1 * t13 * t308 - 0.12e2 * t300 - t45 * t313 + 0.4e1 * t1 * t41 + 0.2e1 * t1 * t48 + t13 * t140 - t76 * t212 + t76 * t321 - 0.2e1 * t39 * t324 + 0.2e1 * t39 * t280;
		t336 = t41 * t12;
		t342 = t70 * t196;
		t344 = t219 * t196;
		t346 = t276 * t71;
		t350 = t188 * t15;
		t359 = -0.4e1 * t6 * t179 + 0.4e1 * t124 + 0.4e1 * t1 * t336 - 0.4e1 * t189 + 0.2e1 * t76 * t250 + 0.4e1 * t342 + 0.8e1 * t344 + 0.2e1 * t346 - 0.2e1 * t1 * t82 - 0.2e1 * t46 * t350 + 0.2e1 * t39 * t146 - 0.2e1 * t6 * t244 + 0.4e1 * t1 * t196;
		t365 = t230 * t196;
		t367 = t159 * t16;
		t369 = t34 * t23;
		t370 = t369 * t71;
		t374 = t369 * t196;
		t376 = t54 * t71;
		t378 = t54 * t196;
		t380 = t178 * t119;
		t382 = t211 * t196;
		t385 = t216 * t196;
		t387 = -0.12e2 * t205 + 0.4e1 * t45 * t250 - t14 * t86 - 0.8e1 * t59 + 0.16e2 * t365 - 0.12e2 * t367 - 0.2e1 * t370 + 0.4e1 * t46 * t244 - 0.4e1 * t374 - 0.2e1 * t376 - 0.4e1 * t378 + 0.8e1 * t380 + 0.2e1 * t139 * t382 + 0.8e1 * t385;
		t389 = t7 * t47;
		t390 = t389 * t71;
		t392 = t34 * t12;
		t393 = t392 * t47;
		t396 = t389 * t196;
		t408 = t178 * t15;
		t411 = t7 * t21;
		t412 = t411 * t47;
		t415 = t34 * t20;
		t416 = t415 * t12;
		t422 = 0.4e1 * t390 + 0.6e1 * t6 * t393 + 0.8e1 * t396 + 0.2e1 * t139 * t308 - 0.2e1 * t46 * t83 + 0.4e1 * t94 * t382 + 0.2e1 * t40 * t86 - 0.4e1 * t6 * t344 - 0.4e1 * t6 * t408 + 0.2e1 * t6 * t412 - t6 * t416 - 0.2e1 * t1 * t350 - 0.2e1 * t118 * t169;
		t423 = t123 * t15;
		t428 = t279 * t196;
		t452 = -0.2e1 * t40 * t423 + 0.2e1 * t45 * t253 - 0.2e1 * t139 * t428 - 0.4e1 * t6 * t42 - 0.4e1 * t94 * t428 + 0.4e1 * t108 + 0.2e1 * t46 * t86 - 0.2e1 * t5 * t324 + t5 * t141 + 0.4e1 * t6 * t151 + 0.4e1 * t5 * t258 * t77 - 0.2e1 * t45 * t324 - 0.6e1 * t259 * t67 + 0.2e1 * t45 * t280;
		t461 = t57 * t15;
		t473 = t411 * t26;
		t476 = t140 * t23;
		t484 = 0.4e1 * t112 * t280 - 0.4e1 * t112 * t324 - 0.2e1 * t112 * t313 + 0.4e1 * t6 * t461 - 0.2e1 * t6 * t378 - 0.6e1 * t40 * t235 - t5 * t258 * t20 + 0.6e1 * t5 * t258 * t12 + 0.4e1 * t6 * t473 - t13 * t476 + 0.2e1 * t94 * t141 + 0.2e1 * t5 * t258 * t26 + 0.2e1 * t369;
		t485 = t35 * t23;
		t491 = t8 * t20;
		t492 = t491 * t12;
		t500 = M_PI * t9 * t15;
		t511 = t216 * t20;
		t517 = 0.4e1 * t6 * t485 + t14 * t130 - 0.6e1 * t40 * t300 - t6 * t492 - 0.2e1 * t5 * t64 * t20 - 0.2e1 * t6 * t374 - 0.2e1 * t39 * t500 - 0.6e1 * t40 * t241 - 0.4e1 * t6 * t231 - 0.4e1 * t6 * t380 - 0.6e1 * t46 * t300 - 0.2e1 * t6 * t511 + 0.8e1 * t46 * t42 + 0.12e2 * t392;
		t529 = t1 * t21;
		t536 = M_PI * t8;
		t537 = t5 * t536;
		t544 = 0.8e1 * t112 * t250 - 0.24e2 * t57 * t238 + t39 * t313 + 0.2e1 * t48 + 0.4e1 * t46 * t336 + 0.8e1 * t178 * t120 + t138 * t529 - 0.8e1 * t57 * t169 - 0.2e1 * t40 * t350 - t6 * t376 + 0.6e1 * t537 * t67 - 0.4e1 * t6 * t385 - 0.2e1 * t6 * t167;
		t551 = t70 * t20;
		t553 = t276 * t20;
		t556 = t389 * t20;
		t573 = -0.12e2 * t159 * t238 - 0.4e1 * t10 * t169 + 0.4e1 * t100 * t120 + 0.2e1 * t551 + 0.2e1 * t553 + 0.4e1 * t294 + 0.4e1 * t556 - t5 * t262 - 0.6e1 * t76 * t88 * t47 + 0.4e1 * t113 * t86 + 0.6e1 * t537 * t300 + 0.4e1 * t76 * t95 * t23 + 0.4e1 * t40 * t336 + 0.4e1 * t40 * t49;
		t580 = t411 * t23;
		t600 = t146 * t11;
		t605 = -0.4e1 * t94 * t250 + 0.2e1 * t6 * t580 - 0.12e2 * t10 * t120 + 0.4e1 * t94 * t308 - 0.4e1 * t113 * t83 + 0.4e1 * t5 * t258 * t11 - 0.2e1 * t1 * t193 + 0.2e1 * t71 - t6 * t370 - t75 * t1 * t266 + t75 * t1 * t23 + 0.4e1 * t45 * t600 + 0.2e1 * t537 * t189;
		t611 = t369 * t20;
		t626 = -0.2e1 * t1 * t99 * t120 - 0.2e1 * t6 * t423 - t6 * t611 + t13 * t253 - 0.8e1 * t461 - 0.2e1 * t55 - 0.2e1 * t611 - 0.2e1 * t416 - 0.2e1 * t46 * t189 + 0.2e1 * t267 - 0.12e2 * t17 - 0.2e1 * t14 * t336 - t6 * t346 - t6 * t130;
		t638 = t34 * t77;
		t639 = t638 * t26;
		t646 = t230 * t20;
		t651 = -0.4e1 * t113 * t423 - 0.4e1 * t139 * t96 - 0.2e1 * t139 * t255 - 0.24e2 * t57 * t120 + 0.2e1 * t1 * t130 + 0.8e1 * t6 * t639 + 0.2e1 * t45 * t262 + 0.4e1 * t511 - 0.8e1 * t473 + 0.8e1 * t646 - 0.12e2 * t241 + 0.4e1 * t193 + 0.4e1 * t423;
		t652 = t267 * t47;
		t654 = t638 * t47;
		t657 = t415 * t11;
		t659 = t491 * t11;
		t662 = t392 * t26;
		t664 = t173 * t15;
		t673 = 0.2e1 * t652 + 0.8e1 * t654 + 0.8e1 * t109 - 0.4e1 * t657 + 0.4e1 * t659 + 0.16e2 * t36 + 0.24e2 * t662 + 0.4e1 * t664 + 0.8e1 * t485 + 0.8e1 * t208 + 0.2e1 * t1 * t20 + 0.2e1 * t86 + 0.4e1 * t49 - 0.2e1 * t415;
		t699 = t392 * t23;
		t702 = 0.12e2 * t6 * t662 - 0.2e1 * t5 * t250 - 0.2e1 * t6 * t217 - 0.4e1 * t14 * t42 - 0.12e2 * t113 * t235 - 0.12e2 * t113 * t300 - 0.12e2 * t159 * t120 - t139 * t476 - 0.2e1 * t6 * t659 - 0.2e1 * t6 * t336 - 0.2e1 * t76 * t382 + 0.2e1 * t76 * t428 + 0.6e1 * t6 * t699;
		t727 = t104 * t20;
		t730 = -t139 * t262 + 0.2e1 * t6 * t225 - 0.4e1 * t6 * t646 + 0.2e1 * t39 * t262 + 0.4e1 * t39 * t600 + 0.2e1 * t6 * t83 - 0.2e1 * t94 * t321 - 0.4e1 * t75 * t1 * t11 + 0.2e1 * t492 + 0.8e1 * t35 + 0.4e1 * t1 * t49 + 0.2e1 * t5 * t500 + 0.2e1 * t5 * t308 - 0.2e1 * t6 * t727;
		t734 = t22 * t47;
		t755 = 0.8e1 * t113 * t244 + t6 * t734 + 0.2e1 * t491 - 0.2e1 * t46 * t124 - 0.2e1 * t1 * t83 - 0.2e1 * t22 + 0.4e1 * t6 * t654 + 0.2e1 * t1 * t86 - 0.2e1 * t537 * t124 + 0.4e1 * t196 - t5 * t536 * t20 - 0.12e2 * t113 * t241 + 0.6e1 * t6 * t241;
		t756 = t108 * t12;
		t783 = -0.2e1 * t6 * t756 - 0.4e1 * t113 * t124 - 0.2e1 * t21 - 0.6e1 * t259 * t300 - 0.12e2 * t113 * t67 - 0.2e1 * t40 * t189 - t139 * t280 + 0.4e1 * t41 - 0.2e1 * t21 * t47 - 0.4e1 * t76 * t95 * t47 - t139 * t321 - 0.2e1 * t6 * t193 - 0.4e1 * t75 * t1 * t77 - 0.2e1 * t6 * t220;
		t795 = M_PI * t266;
		t806 = t638 * t23;
		t809 = t267 * t26;
		t811 = -0.2e1 * t40 * t193 + 0.8e1 * t638 - 0.2e1 * t6 * t165 + 0.2e1 * t34 - 0.2e1 * t259 * t189 - t76 * t795 * t47 - 0.4e1 * t21 * t26 - 0.4e1 * t113 * t350 + 0.2e1 * t94 * t253 + 0.2e1 * t39 * t253 + 0.8e1 * t806 + 0.12e2 * t699 + 0.4e1 * t809;
		t816 = t100 * t15;
		t823 = t159 * t15;
		t828 = -0.4e1 * t727 + 0.4e1 * t756 + 0.4e1 * t336 + 0.8e1 * t42 + 0.4e1 * t816 + 0.16e2 * t639 - 0.4e1 * t412 - 0.4e1 * t580 - 0.2e1 * t24 + 0.8e1 * t408 - 0.4e1 * t823 - 0.12e2 * t235 - 0.4e1 * t350 - 0.4e1 * t30;
		t849 = 0.2e1 * t268 + 0.12e2 * t393 - 0.2e1 * t734 - 0.4e1 * t83 - 0.4e1 * t27 + 0.4e1 * t244 + 0.2e1 * t130 - 0.4e1 * t112 * t500 + 0.4e1 * t65 * t189 - 0.4e1 * t65 * t124 - 0.2e1 * t94 * t476 - 0.6e1 * t6 * t367 + 0.8e1 * t1 * t42;
		t877 = 0.4e1 * t39 * t250 + 0.2e1 * t6 * t189 + 0.2e1 * t259 * t124 - 0.2e1 * t6 * t556 + t76 * t795 * t23 - 0.2e1 * t76 * t795 * t26 - 0.2e1 * t21 * t23 - t6 * t551 - t6 * t553 + 0.2e1 * t5 * t64 * t21 - 0.4e1 * t6 * t197 + 0.4e1 * t113 * t130 + 0.4e1 * t192 + 0.2e1 * t40 * t115;
		t904 = t5 * t476 - t13 * t262 - 0.4e1 * t6 * t396 + 0.4e1 * t76 * t78 * t23 + 0.4e1 * t104 - 0.2e1 * t14 * t49 - 0.8e1 * t6 * t365 + 0.2e1 * t14 * t244 - 0.2e1 * t6 * t390 - 0.6e1 * t46 * t241 - 0.2e1 * t6 * t342 + 0.2e1 * t93 * t529 - 0.4e1 * t94 * t255;
		t933 = 0.2e1 * t6 * t350 + 0.2e1 * t6 * t809 + 0.6e1 * t6 * t235 - 0.2e1 * t6 * t277 - 0.2e1 * t45 * t500 - 0.2e1 * t94 * t280 + 0.8e1 * t112 * t600 + 0.4e1 * t1 * t244 - 0.2e1 * t46 * t115 - t5 * t253 - 0.2e1 * t6 * t657 - 0.6e1 * t6 * t160 - 0.2e1 * t75 * t1 * t26 + 0.12e2 * t65 * t300;
		t962 = -0.2e1 * t94 * t262 + 0.2e1 * t6 * t664 + t5 * t536 * t21 - 0.2e1 * t1 * t423 + 0.4e1 * t6 * t806 + 0.2e1 * t6 * t186 - 0.2e1 * t6 * t816 - 0.6e1 * t46 * t67 - 0.2e1 * t13 * t250 + 0.8e1 * t113 * t336 + 0.4e1 * t112 * t262 - 0.6e1 * t40 * t67 - 0.8e1 * t76 * t95 * t26;
		t988 = t5 * t258 * t266 + t5 * t258 * t47 - 0.4e1 * t411 + t5 * t258 * t23 + t6 * t652 - 0.4e1 * t76 * t78 * t47 - 0.2e1 * t46 * t423 - 0.2e1 * t46 * t193 - t76 * t253 + t76 * t280 + 0.8e1 * t113 * t49 - 0.2e1 * t5 * t600 - 0.2e1 * t6 * t823 + 0.2e1 * t46 * t130;
		t1001 = pow(t9 + 0.1e1 + t11, 0.2e1);
		t1004 = pow(t9 - 0.1e1 - t11, 0.2e1);
		t1011 = n * n;
		t1015 = M_PI * M_PI;

		u3b = -deltaRho * t3 * (t74 + t933 + t544 + t299 + t164 + t849 + t904 + t702 + t651 + t452 + t828 + t199 + t387 + t359 + t962 + t133 + t877 + t811 + t484 + t422 + t605 + t783 + t730 + t626 + t272 + t673 + t573 + t233 + t329 + t755 + t988 + t517) / t15 / (0.1e1 + t26) / t1001 / t1004 / t33 / (0.1e1 + t7) / t1011 / n / t1015 / M_PI / 0.2e1;

		if ( y > startBlockY ) {
			u2=u2b;u3=u3b;
		}
		else {
			u2=u2a;u3=u3a;
		}

		u5 = (double)-2*n*M_PI*(viscosity*u2 + u3)*cos(n*M_PI*x);

		*pressure +=u5;

	}
	
	if ( y > startBlockY ) {
	    *pressure += deltaRho*dx*(0.5-y);
	 } else {
	    *pressure += deltaRho*dx*(0.5-startBlockY);
	}

}

void _AnalyticBlock_AssignFromXML( void* analyticSolution, Stg_ComponentFactory* cf, void* data ) {
	AnalyticBlock*           self           = (AnalyticBlock*)analyticSolution;

	/* Construct Parent */
	_AnalyticSolution_AssignFromXML( self, cf, data );

	/* Create Analytic Fields */
	self->velocityField = Stg_ComponentFactory_ConstructByName( cf, (Name)"VelocityField", FeVariable, True, data  ); 
	self->pressureField = Stg_ComponentFactory_ConstructByName( cf, (Name)"PressureField", FeVariable, True, data  ); 

	self->startBlockX = Stg_ComponentFactory_GetRootDictDouble( cf, (Dictionary_Entry_Key)"startBlockX", 0.0  );
	self->endBlockX = Stg_ComponentFactory_GetRootDictDouble( cf, (Dictionary_Entry_Key)"endBlockX", 0.0  );
	self->startBlockY = Stg_ComponentFactory_GetRootDictDouble( cf, (Dictionary_Entry_Key)"startBlockY", 0.0  );
}

void _AnalyticBlock_Build( void* analyticSolution, void* data ) {
	AnalyticBlock*	self = (AnalyticBlock*)analyticSolution;

	assert( self && Stg_CheckType( self, AnalyticBlock ) );

	Build( self->velocityField, data, False );
	Build( self->pressureField, data, False );
	AnalyticSolution_CreateAnalyticField( self, self->velocityField, _AnalyticBlock_VelocityFunction );
	AnalyticSolution_CreateAnalyticField( self, self->pressureField, _AnalyticBlock_PressureFunction );

	_AnalyticSolution_Build( self, data );
}

void* _AnalyticBlock_DefaultNew( Name name ) {
	/* Variables set in this function */
	SizeT                                              _sizeOfSelf = sizeof(AnalyticBlock);
	Type                                                      type = AnalyticBlock_Type;
	Stg_Class_DeleteFunction*                              _delete = _AnalyticSolution_Delete;
	Stg_Class_PrintFunction*                                _print = _AnalyticSolution_Print;
	Stg_Class_CopyFunction*                                  _copy = _AnalyticSolution_Copy;
	Stg_Component_DefaultConstructorFunction*  _defaultConstructor = _AnalyticBlock_DefaultNew;
	Stg_Component_ConstructFunction*                    _construct = _AnalyticBlock_AssignFromXML;
	Stg_Component_BuildFunction*                            _build = _AnalyticBlock_Build;
	Stg_Component_InitialiseFunction*                  _initialise = _AnalyticSolution_Initialise;
	Stg_Component_ExecuteFunction*                        _execute = _AnalyticSolution_Execute;
	Stg_Component_DestroyFunction*                        _destroy = _AnalyticSolution_Destroy;

	/* Variables that are set to ZERO are variables that will be set either by the current _New function or another parent _New function further up the hierachy */
	AllocationType  nameAllocationType = NON_GLOBAL /* default value NON_GLOBAL */;

	return _AnalyticSolution_New(  ANALYTICSOLUTION_PASSARGS  );
}

Index _PICellerator_AnalyticBlock_Register( PluginsManager* pluginsManager ) {
	return PluginsManager_Submit( pluginsManager, AnalyticBlock_Type, (Name)"0", _AnalyticBlock_DefaultNew  );
}


