/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
**
** Copyright (C), 2003, Victorian Partnership for Advanced Computing (VPAC) Ltd, 110 Victoria Street, Melbourne, 3053, Australia.
**
** Authors:
**	Stevan M. Quenette, Senior Software Engineer, VPAC. (steve@vpac.org)
**	Patrick D. Sunter, Software Engineer, VPAC. (pds@vpac.org)
**	Luke J. Hodkinson, Computational Engineer, VPAC. (lhodkins@vpac.org)
**	Siew-Ching Tan, Software Engineer, VPAC. (siew@vpac.org)
**	Alan H. Lo, Computational Engineer, VPAC. (alan@vpac.org)
**	Raquibul Hassan, Computational Engineer, VPAC. (raq@vpac.org)
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
**
**
**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

#include "Base/Foundation/Foundation.h"

#include "types.h"
#include "shortcuts.h"
#include "Journal.h"
#include "StreamFormatter.h"
#include "Finalise.h"
#include "IO.h"


#include <stdio.h>

Bool BaseIO_Finalise( void )
{
	Stream* stream;
	if ( stJournal->flushCount > 0 ) {
		stream = Journal_Register( Debug_Type, (char*)__func__ );
		Journal_Printf( stream, "StGermain IO Report - File Flush called %d times.\n", stJournal->flushCount );
	}
	Journal_Delete();
	stJournal = NULL;
	/* Now clean up the special memory streams seeing as these aren't in the journal */
	Stg_Class_Delete( stgMemory->infoStream );
	stgMemory->infoStream = NULL;
	Stg_Class_Delete( stgMemory->debugStream );
	stgMemory->debugStream = NULL;
	Stg_Class_Delete( stgMemory->errorStream );
	stgMemory->errorStream = NULL;
	StreamFormatter_Buffer_Delete( stgStreamFormatter_Buffer );
	stgStreamFormatter_Buffer = NULL;
	Stg_Class_Delete( _stgFilePaths );
	_stgFilePaths = NULL;
	Stg_Class_Delete( Project_XMLSearchPaths );
	Project_XMLSearchPaths = NULL;
	
	return True;
}


